/**
 * 
 */
package de.digisalt.dsnesds.core.model.objects;

import java.io.IOException;
import java.math.BigInteger;
import java.util.Vector;

import org.eclipse.core.internal.resources.ResourceException;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;

import de.digisalt.dsnesds.core.model.Activator;

/**
 * @author flo
 * 
 */
@SuppressWarnings("restriction")
public class PaletteObject
{
	
	/**
	 * the number of colors supported by this palette
	 */
	private final int numOfColors = 16;
	/**
	 * all red values
	 */
	private final int[] r = new int[16];
	/**
	 * all green values
	 */
	private final int[] g = new int[16];
	/**
	 * all blue values
	 */
	private final int[] b = new int[16];
	
	/**
	 * @param res
	 *          the Resource the paletteObject is being created by
	 * @param randomTest
	 *          true if we want to mock loading of the data, falso otherwise
	 * @throws ResourceException
	 *           if the resource couldn't be read
	 * @throws IOException
	 *           if the file couldn't be read
	 */
	public PaletteObject(final IResource res, final boolean randomTest)
			throws ResourceException, IOException
	{
		// load the resource content into the pojo
		if (!(res instanceof IFile))
		{
			throw new ResourceException(new Status(IStatus.ERROR, Activator.id,
					"Can't load palette " + res.getName() + " as it is no file."));
		}
		if (!res.exists())
		{
			throw new ResourceException(new Status(IStatus.ERROR, Activator.id,
					"Can't load palette " + res.getName()
							+ " as the corresponding file seems to be not existing."));
		}
		IFile input = (IFile) res;
		
		// get palette content
		// File pal = ResourceUtil.getFileForIFile(input);
		
		// create array of colors
		// this.numOfColors = (int) (pal.length() / 2);
		// this.r = new int[this.numOfColors];
		// this.g = new int[this.numOfColors];
		// this.b = new int[this.numOfColors];
		
		// for (int i = 0; i < this.numOfColors; i++)
		// {
		// this.r[i] = 0;
		// this.g[i] = 0;
		// this.b[i] = 0;
		// }
		//
		// Random randGen = new Random(System.currentTimeMillis());
		// if (randomTest && this.numOfColors != 32768)
		// {
		// for (int i = 0; i < this.numOfColors; i++)
		// {
		// this.r[i] = Math.abs(randGen.nextInt()) % 255;
		// this.g[i] = Math.abs(randGen.nextInt()) % 255;
		// this.b[i] = Math.abs(randGen.nextInt()) % 255;
		// }
		// } else if (randomTest && this.numOfColors == 32768)
		// {
		// // create a palette covering the complete snes colorspace
		// int i = 0;
		// for (int rv = 0; rv <= 31; rv++)
		// {
		// for (int gv = 0; gv <= 31; gv++)
		// {
		// for (int bv = 0; bv <= 31; bv++)
		// {
		// this.r[i] = rv;
		// this.g[i] = gv;
		// this.b[i] = bv;
		// i++;
		// }
		// }
		// }
		// } else
		// {
		// // fill with resource content
		// FileInputStream s = ResourceUtil.createFileInputStream(pal);
		//
		// byte[] b = new byte[this.numOfColors * 2];
		//
		// s.read(b, 0, this.numOfColors * 2);
		//
		// setByteContent(b);
		// }
	}
	
	/**
	 * This method uses an byte array to generate the colors
	 * 
	 * @param bytes
	 *          the byte array
	 * @return true if everything went alright, false otherwise
	 */
	private boolean setByteContent(final byte[] bytes)
	{
		for (int i = 0; i < this.numOfColors; i++)
		{
			byte one = bytes[(i * 2) + 0];
			byte two = bytes[(i * 2) + 1];
			int[] rgb = new int[3];
			rgb = getRawSnesColor(one, two);
			this.r[i] = rgb[0] * 8;
			this.g[i] = rgb[1] * 8;
			this.b[i] = rgb[2] * 8;
		}
		return true;
	}
	
	/**
	 * takes the byte one and byte two (?gggggbb bbbrrrrr) and makes and integer
	 * array out of it --> int[] {0..31, 0..31, 0..31}
	 * 
	 * @param one
	 *          the first byte containing the color data
	 * @param two
	 *          the second byte containing the color data
	 * @return an int[] containing {red, green, blue}
	 */
	private int[] getRawSnesColor(final byte one, final byte two)
	{
		BigInteger bitString = new BigInteger(new byte[]
		{ one, two });
		// System.out.println(bitString.toString(2));
		BigInteger rred = bitString.and(new BigInteger("0000000000011111", 2));
		BigInteger ggreen = bitString.and(new BigInteger("0000001111100000", 2));
		BigInteger bblue = bitString.and(new BigInteger("0111110000000000", 2));
		String redS = rred.toString(2);
		String greenS = ggreen.toString(2);
		String blueS = bblue.toString(2);
		// expand binary strings to exactly 15 chars
		while (redS.length() < 15)
		{
			redS = "0" + redS;
		}
		while (greenS.length() < 15)
		{
			greenS = "0" + greenS;
		}
		while (blueS.length() < 15)
		{
			blueS = "0" + blueS;
		}
		// get relevant bits from binary strings --> binary strings of length 5
		blueS = blueS.substring(0, 5);
		greenS = greenS.substring(5, 10);
		redS = redS.substring(10, 15);
		// generate Integer objects out of the bitstrings
		Integer redI = Integer.parseInt(redS, 2);
		Integer greenI = Integer.parseInt(greenS, 2);
		Integer blueI = Integer.parseInt(blueS, 2);
		// generate three integers out of the binary strings
		int[] rgb = new int[3];
		rgb[0] = redI.intValue();
		rgb[1] = greenI.intValue();
		rgb[2] = blueI.intValue();
		// System.out.println(rgb[0]+"|"+rgb[1]+"|"+rgb[2]);
		return rgb;
	}
	
	/**
	 * colNum = [0..numOfColors-1], red,green,blue = [0..255]
	 * 
	 * @param colNum
	 *          the number of the color (0 .. N-1)
	 * @return the color as an int array {red, green, blue}
	 */
	public int[] getColor(final int colNum)
	{
		if (colNum > this.numOfColors - 1 || colNum < 0)
		{
			return new int[] {};
		}
		int[] res = new int[3];
		res[0] = this.r[colNum];
		res[1] = this.g[colNum];
		res[2] = this.b[colNum];
		return res;
	}
	
	/**
	 * returns the number of colors this palette holds
	 * 
	 * @return the number of colors this palette holds
	 */
	public int getNumberOfColors()
	{
		return this.numOfColors;
	}
	
	/**
	 * This method returns the content of the PaletteObject as a Byte-array. This
	 * is needed for saving the palette as a separate file
	 * 
	 * @return a byte[] containing all byte content of the file
	 */
	public byte[] getByteContent()
	{
		// get output like this sample:
		// [00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00]
		Vector<Byte> v = new Vector<Byte>();
		int r1, g1, b1 = 0;
		
		for (int i = 0; i < this.numOfColors; i++)
		{
			if (i != 1)
			{
			}
			r1 = this.r[i];
			g1 = this.g[i];
			b1 = this.b[i];
			// make two binary strings out of the colors
			String[] binStr = makeSnesColor(r1, g1, b1).split(" ");
			// convert binary strings to two hexbytes
			v.add(new Byte(makeByte(binStr[0])));
			v.add(new Byte(makeByte(binStr[1])));
		}
		byte[] b = new byte[v.size()];
		for (int i = 0; i < v.size(); i++)
		{
			b[i] = v.get(i).byteValue();
		}
		return b;
	}
	
	/**
	 * will return an WLA-DX compatible string of this palette
	 * 
	 * @return something like ".db $00, $00, ..."
	 */
	public String getAsmContent()
	{
		// get output like this sample:
		// .db $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00,
		// $00, $00
		String ret = ".db ";
		int[] rgb = new int[] {};
		for (int i = 1; i <= this.numOfColors; i++)
		{
			if (i != 1)
			{
				ret = ret + ",";
			}
			// get color number i-1
			rgb = getColor(i - 1);
			// System.out.println(rgb[0] + "|" + rgb[1] + "|" + rgb[2]);
			// make two binary strings out of the colors
			String[] binStr = makeSnesColor(rgb[0] / 8, rgb[1] / 8, rgb[2] / 8)
					.split(" ");
			// convert binary strings to two hexbytes
			ret = ret + " $" + makeHexString(binStr[0]) + ", $"
					+ makeHexString(binStr[1]);
		}
		// System.out.println(ret);
		return ret;
	}
	
	/**
	 * Will create a binary string representing the color in SNES format
	 * 
	 * @param r
	 *          the original red value
	 * @param g
	 *          the original green value
	 * @param b
	 *          the original blue value
	 * @return the binary string "0100110101101010" (or something similar)
	 */
	private String makeSnesColor(final int r, final int g, final int b)
	{
		String rs = makeBinaryString(r);
		String gs = makeBinaryString(g);
		String bs = makeBinaryString(b);
		// ?bbbbbgg gggrrrrr
		String bin = "0/" + bs + "/" + gs.substring(0, 2) + " " + gs.substring(2)
				+ "/" + rs;
		// System.out.println(bin);
		bin = "0" + bs + gs.substring(0, 2) + " " + gs.substring(2) + rs;
		return bin;
	}
	
	// works correctly
	private String makeBinaryString(int i)
	{
		// ignore all values bigger than 31
		if (i > 31)
		{
			i = 31;
		}
		// ignore all values less than 0
		if (i < 0)
		{
			i = 0;
		}
		// create binary string out of integer
		BigInteger t = new BigInteger("" + i);
		String ret = t.toString(2);
		// make sure it is 5 digits long
		while (ret.length() < 5)
		{
			ret = "0" + ret;
		}
		// System.out.println(i + " == " + ret + "?!");
		return ret;
	}
	
	private String makeHexString(final String binStr)
	{
		BigInteger bi = new BigInteger(binStr, 2);
		String s = bi.toString(16);
		if (s.length() % 2 != 0)
		{
			// Pad with 0
			s = "0" + s;
		}
		return s;
	}
	
	private byte makeByte(final String binStr)
	{
		BigInteger bi = new BigInteger(binStr, 2);
		return bi.byteValue();
	}
}
