/**
 * 
 */
package de.digisalt.dsnesds.core.model.util;

import java.util.HashMap;

import org.eclipse.core.internal.resources.ResourceException;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;

import de.digisalt.dsnesds.core.model.objects.DsnesdsProjectRoot;

/**
 * @author flo
 * 
 */
public class ProjectModelUtil
{
	/**
	 * Contains a list of all IProject-DsnesdsProjectRoot objects
	 */
	private static HashMap<IProject, DsnesdsProjectRoot> allProjects = new HashMap<IProject, DsnesdsProjectRoot>();
	
	/**
	 * returns the DsnesdsProjectRoot for the given IProject. If there wasn't a
	 * DsnesdsProjectRoot for the given IProject before, it is created
	 * 
	 * @param project
	 *          the IProject to get the DSnesdsProjectRoot for
	 * @return the DsnedssProjectRoot
	 * @throws CoreException
	 *           may occur if anything goes wrong reading the project resources
	 */
	public static DsnesdsProjectRoot getProjectRoot(final IProject project)
			throws CoreException
	{
		// do we already have a DsnesdsProjectRoot for the given IProject?
		if (allProjects.get(project) != null)
		{
			// yes --> return it
			return allProjects.get(project);
		}
		// no --> create it, add to HashMap and return it
		DsnesdsProjectRoot newRoot = new DsnesdsProjectRoot(project);
		
		IResource[] projectResources = project.members();
		addAllResourceContentToProject(newRoot, projectResources);
		
		allProjects.put(project, newRoot);
		return newRoot;
	}
	
	/**
	 * Needs to be invoked by the project builder when a full build is done. Will
	 * remove the complete model reference from the list
	 * 
	 * @param project
	 *          the IProject to remove the DsnesdsProjectRoot for
	 * @return the DsnedsProjectRoot created
	 * @throws CoreException
	 *           may occur if anything goes wrong reading the resources of the
	 *           project
	 */
	public static DsnesdsProjectRoot recreateProjectRoot(final IProject project)
			throws CoreException
	{
		allProjects.remove(project);
		return getProjectRoot(project);
	}
	
	/**
	 * @param projectRoot
	 *          the project root to add the resources pojos to
	 * @param resources
	 *          the resource array holding all resources
	 * @throws ResourceException
	 */
	private static void addAllResourceContentToProject(
			final DsnesdsProjectRoot projectRoot, final IResource[] resources)
			throws ResourceException
	{
		for (IResource res : resources)
		{
			if (res instanceof IFile)
			{
				IFile ifile = (IFile) res;
				String ext = ifile.getFileExtension();
				if (ext.equalsIgnoreCase("pal"))
				{
					// palette found --> add it to the list of palettes
					// PaletteObject pal = new PaletteObject(ifile, false);
					// projectRoot.addPalette(pal);
				}
			}
		}
	}
}
