/**
 * 
 */
package de.digisalt.dsnesds.core.utils.palette;

import java.math.BigInteger;
import java.util.Vector;

/**
 * @author flo
 * 
 */
public class PaletteUtils
{
	/**
	 * takes the byte one and byte two (?gggggbb bbbrrrrr) and creates an integer
	 * array out of it containing the red green and blue values (in this order)
	 * with values between 0 and 31
	 * 
	 * @param one
	 *          first byte of the color
	 * @param two
	 *          second byte of the color
	 * @return the integer array containing three ints (0..31) for red green and
	 *         blue
	 */
	public static int[] getRawSnesColor(final byte one, final byte two)
	{
		BigInteger bitString = new BigInteger(new byte[]
		{ one, two });
		// System.out.println(bitString.toString(2));
		BigInteger rred = bitString.and(new BigInteger("0000000000011111", 2));
		BigInteger ggreen = bitString.and(new BigInteger("0000001111100000", 2));
		BigInteger bblue = bitString.and(new BigInteger("0111110000000000", 2));
		String redS = rred.toString(2);
		String greenS = ggreen.toString(2);
		String blueS = bblue.toString(2);
		// expand binary strings to exactly 15 chars
		while (redS.length() < 15)
		{
			redS = "0" + redS;
		}
		while (greenS.length() < 15)
		{
			greenS = "0" + greenS;
		}
		while (blueS.length() < 15)
		{
			blueS = "0" + blueS;
		}
		// get relevant bits from binary strings --> binary strings of length 5
		blueS = blueS.substring(0, 5);
		greenS = greenS.substring(5, 10);
		redS = redS.substring(10, 15);
		// generate Integer objects out of the bitstrings
		Integer redI = Integer.parseInt(redS, 2);
		Integer greenI = Integer.parseInt(greenS, 2);
		Integer blueI = Integer.parseInt(blueS, 2);
		// generate three integers out of the binary strings
		int[] rgb = new int[3];
		rgb[0] = redI.intValue();
		rgb[1] = greenI.intValue();
		rgb[2] = blueI.intValue();
		return rgb;
	}
	
	/**
	 * Takes three integers between 0 and 31 for red, green and blue and makes two
	 * bytes out of them which can be serialized
	 * 
	 * @param r
	 *          the red input value (0..31)
	 * @param g
	 *          the green input value (0..31)
	 * @param b
	 *          the blue input value (0..31)
	 * @return the byte array containing the two bytes in the following format
	 *         (?gggggbb bbbrrrrr)
	 */
	public static byte[] getByteContent(final int r, final int g, final int b)
	{
		Vector<Byte> v = new Vector<Byte>();
		// make two binary strings out of the colors
		String[] binStr = makeSnesColor(r, g, b).split(" ");
		// convert binary strings to two hexbytes
		v.add(new Byte(makeByte(binStr[0])));
		v.add(new Byte(makeByte(binStr[1])));
		byte[] bytes = new byte[2];
		
		for (int i = 0; i < v.size(); i++)
		{
			bytes[i] = v.get(i).byteValue();
		}
		return bytes;
	}
	
	/**
	 * takes three integers (0..31) and creates a string of the following format
	 * out of them: "0gggggbb bbbrrrrr"
	 * 
	 * @param r
	 *          the red input value (0..31)
	 * @param g
	 *          the green input value (0..31)
	 * @param b
	 *          the blue input value (0..31)
	 * @return the binary string representation of the colors
	 */
	private static String makeSnesColor(final int r, final int g, final int b)
	{
		String rs = makeBinaryString(r);
		String gs = makeBinaryString(g);
		String bs = makeBinaryString(b);
		// ?bbbbbgg gggrrrrr
		String bin = "0/" + bs + "/" + gs.substring(0, 2) + " " + gs.substring(2)
				+ "/" + rs;
		// System.out.println(bin);
		bin = "0" + bs + gs.substring(0, 2) + " " + gs.substring(2) + rs;
		return bin;
	}
	
	/**
	 * Takes an input number (0..31) and creates a binary String out of that. The
	 * string is always five digits long. In case a number outside the range
	 * (0..31) is used as input, its nearest neighbor is used instead.
	 * 
	 * @param i
	 *          the integer to be converted to five digit binary (expected to be
	 *          0..31)
	 * @return the five character representation of the binary
	 */
	private static String makeBinaryString(int i)
	{
		// ignore all values bigger than 31
		if (i > 31)
		{
			i = 31;
		}
		// ignore all values less than 0
		if (i < 0)
		{
			i = 0;
		}
		// create binary string out of integer
		BigInteger t = new BigInteger("" + i);
		String ret = t.toString(2);
		// make sure it is 5 digits long
		while (ret.length() < 5)
		{
			ret = "0" + ret;
		}
		// System.out.println(i + " == " + ret + "?!");
		return ret;
	}
	
	/**
	 * Converts a binary string into a byte
	 * 
	 * @param binStr
	 *          the binary input string
	 * @return the corresponding byte value
	 */
	private static byte makeByte(final String binStr)
	{
		BigInteger bi = new BigInteger(binStr, 2);
		return bi.byteValue();
	}
}
