/**
 * 
 */
package de.digisalt.dsnesds.core.wizards.create.file;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;

import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.IWizard;
import org.eclipse.ui.dialogs.WizardNewFileCreationPage;

import de.digisalt.dsnesds.core.wizards.Activator;

/**
 * @author flo
 */
public class AbstractNewFileCreationPage extends WizardNewFileCreationPage
{

	/**
	 * the constructor for the page
	 * 
	 * @param pageName
	 *          the name of the page
	 * @param selection
	 *          the selection with which to initialize the page
	 */
	public AbstractNewFileCreationPage(final String pageName,
			final IStructuredSelection selection)
	{
		super(pageName, selection);
	}

	@Override
	protected InputStream getInitialContents()
	{
		IWizard wizard = getWizard();
		if (wizard instanceof IByteStreamInput)
		{
			// get the byte content to put into the new file
			byte[] bytes = ((IByteStreamInput) wizard).getByteContent();
			return new ByteArrayInputStream(bytes);
		}
		else if (wizard instanceof IStringStreamInput)
		{
			// get the String content to put into the new file
			String content = ((IStringStreamInput) wizard).getStringContent();
			return new ByteArrayInputStream(content.getBytes());
		}
		else if (wizard instanceof IResourceStreamInput)
		{
			// get a bunlde resource as byte stream to write
			String path = ((IResourceStreamInput) wizard).getResourcePath();
			try
			{
				return Activator.getDefault().getBundle().getEntry(path).openStream();
			}
			catch (IOException e)
			{
				e.printStackTrace();
				return null;
			}
		}
		return null;
	}
}
