/**
 * 
 */
package de.digisalt.dsnesds.core.wizards.create.file;

import java.util.HashMap;

import org.eclipse.jface.resource.ImageDescriptor;

/**
 * @author flo
 */
public class NewFileWizardConfigBean
{
	/**
	 * the title of the wizard
	 */
	private String									content			= "File";

	/**
	 * the wizard title image to display
	 */
	private ImageDescriptor					titleImage	= null;

	/**
	 * the file extension to be used for the files created by this wizard
	 */
	private String									extension		= "";

	/**
	 * a hashmap containing key-value pairs of metadata to be associated to the
	 * newly created file
	 */
	private HashMap<String, String>	metadata		= null;

	/**
	 * returns the wizards title
	 * 
	 * @return the wizards title as a string
	 */
	public String getTitle()
	{
		return "New " + this.content;
	}

	/**
	 * returns the String name of the content the wizard can create
	 * 
	 * @return the String name of the content the wizard can create
	 */
	public String getContent()
	{
		return this.content;
	}

	/**
	 * sets the String name of the content, this wizard can create
	 * 
	 * @param content
	 *          the string name of the content this wizard creates
	 */
	public void setContent(final String content)
	{
		this.content = content;
	}

	/**
	 * sets the ImageDescriptor which will be displayed in the head of the wizard
	 * 
	 * @param titleImage
	 *          the imageDescriptor to be used
	 */
	public void setTitleImage(ImageDescriptor titleImage)
	{
		this.titleImage = titleImage;
	}

	/**
	 * retrieve the configured ImageDescriptor (header image) for the Wizard
	 * 
	 * @return the ImageDesciptor for the wizard
	 */
	public ImageDescriptor getTitleImage()
	{
		return this.titleImage;
	}

	/**
	 * This sets the file extension which will be created by the wizard
	 * 
	 * @param ext
	 *          the extension used
	 */
	public void setExtension(String ext)
	{
		this.extension = ext;
	}

	/**
	 * This will return the file extension configured to be created by the wizard
	 * 
	 * @return the extension used
	 */
	public String getExtension()
	{
		return extension;
	}

	public HashMap<String, String> getMetadata()
	{
		return metadata;
	}

	public void setMetadata(HashMap<String, String> metadata)
	{
		this.metadata = metadata;
	}

}
