/**
 * 
 */
package de.digisalt.dsnesds.core.wizards.create.file.palette;

import java.util.Random;

import org.eclipse.wb.swt.ResourceManager;

import de.digisalt.dsnesds.core.utils.palette.PaletteUtils;
import de.digisalt.dsnesds.core.wizards.create.file.AbstractNewFileWizard;
import de.digisalt.dsnesds.core.wizards.create.file.IByteStreamInput;
import de.digisalt.dsnesds.core.wizards.create.file.MetadataObject;
import de.digisalt.dsnesds.core.wizards.create.file.NewFileWizardConfigBean;

/**
 * @author flo
 */
public class NewPaletteWizard extends AbstractNewFileWizard implements
		IByteStreamInput
{
	/**
	 * The second page where the initial setup of the palette is configured
	 */
	private NewPaletteWizardPage	newPaletteWizardPage;

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * de.digisalt.dsnesds.core.wizards.create.file.AbstractNewFileWizard#configure
	 * ()
	 */
	@Override
	protected NewFileWizardConfigBean configure()
	{
		NewFileWizardConfigBean config = new NewFileWizardConfigBean();
		config.setContent("Color Palette");
		config.setTitleImage(ResourceManager.getPluginImageDescriptor(
				"de.digisalt.dsnesds.core.wizards", "graphics/wizards/palette.png"));
		config.setExtension("pal");
		// TODO: add custom configuration here
		return config;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * de.digisalt.dsnesds.core.wizards.create.file.IByteStreamInput#getByteContent
	 * ()
	 */
	@Override
	public byte[] getByteContent()
	{
		// get number of colors the palette should contain
		int colorDepth = this.newPaletteWizardPage.getColorDepth();

		// create byte output array to write to hdd
		byte[] output = new byte[colorDepth * 2];

		// fill the byte array in various ways (user can select to use an existing
		// palette, or to match from an image or from the clipboard or to fill with
		// a single color
		switch (newPaletteWizardPage.getImportMode())
		{
		case "random colors":
		{
			// fill the newly created byte array with random values
			new Random().nextBytes(output);
			break;
		}
		default:
		{
			// default: single color --> will always work as the color is preset in
			// the wizard
			// get RGB to fill palette with
			int red = this.newPaletteWizardPage.getRedValue();
			int green = this.newPaletteWizardPage.getGreenValue();
			int blue = this.newPaletteWizardPage.getBlueValue();
			// create SNES bytes out of that
			byte[] byteContent = PaletteUtils.getByteContent(red, green, blue);
			for (int i = 0; i < colorDepth; i++)
			{
				output[i * 2] = byteContent[0];
				output[(i * 2) + 1] = byteContent[1];
			}
			break;
		}
		}

		// return the filled byte array to be persisted
		return output;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.digisalt.dsnesds.core.wizards.create.file.AbstractNewFileWizard#
	 * addAdditionalPages()
	 */
	@Override
	public void addAdditionalPages()
	{
		this.newPaletteWizardPage = new NewPaletteWizardPage();
		addPage(this.newPaletteWizardPage);
	}

	@Override
	protected MetadataObject getMetadata()
	{
		return null;
		// TODO: if we should need some metadata here uncomment the following code
		// and attack key-value pairs
		// MetadataObject metadata = new MetadataObject(
		// DsnesdsConstants.namespace_paletteFile);
		// return metadata;
	}
}
