/**
 * 
 */
package de.digisalt.dsnesds.core.wizards.create.file.palette;

import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.ColorDialog;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.wb.swt.ResourceManager;
import org.eclipse.wb.swt.SWTResourceManager;

/**
 * @author flo
 */
public class NewPaletteWizardPage extends WizardPage
{
	/**
	 * This combo holds the potential color depths (2, 4, 8, 16 and 256) in this
	 * order
	 */
	private Combo		combo;
	/**
	 * This radiobutton signalizes that the user wants to create a new palette
	 * with a single color initialized in every slot (default behaviour of the
	 * wizard)
	 */
	private Button	btnSingleColor;
	private Label		singleColorDisplay;
	private Button	btnRandomColors;

	/**
	 * Create the wizard.
	 */
	public NewPaletteWizardPage()
	{
		super("wizardPage");
		setImageDescriptor(ResourceManager.getPluginImageDescriptor(
				"de.digisalt.dsnesds.core.wizards", "graphics/wizards/palette.png"));
		setTitle("New color palette");
		setDescription("Create a new color palette with the following presets");
	}

	/**
	 * Create contents of the wizard.
	 * 
	 * @param parent
	 *          the parent to which all content of this page is added
	 */
	@Override
	public void createControl(final Composite parent)
	{
		Composite container = new Composite(parent, SWT.NULL);

		setControl(container);
		container.setLayout(new GridLayout(2, false));

		Label lblNumberOfColors = new Label(container, SWT.NONE);
		lblNumberOfColors.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false,
				false, 1, 1));
		lblNumberOfColors.setText("Number of Colors");

		this.combo = new Combo(container, SWT.READ_ONLY);
		String[] colorDepths =
		{ "2", "4", "8", "16", "256" };
		this.combo.setItems(colorDepths);
		this.combo.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1,
				1));
		this.combo.select(3);

		Group grpInitialContent = new Group(container, SWT.NONE);
		grpInitialContent.setLayout(new GridLayout(3, false));
		grpInitialContent.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true,
				true, 2, 1));
		grpInitialContent.setText("Initial Content");

		this.btnSingleColor = new Button(grpInitialContent, SWT.RADIO);
		this.btnSingleColor.setSelection(true);
		this.btnSingleColor.setText("Single color");

		singleColorDisplay = new Label(grpInitialContent, SWT.BORDER);
		singleColorDisplay.setBackground(SWTResourceManager
				.getColor(SWT.COLOR_TITLE_BACKGROUND));
		singleColorDisplay.setAlignment(SWT.CENTER);
		singleColorDisplay.setText("                              ");
		singleColorDisplay.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false,
				false, 1, 1));

		Button btnSelectColor = new Button(grpInitialContent, SWT.NONE);
		btnSelectColor.addMouseListener(new MouseAdapter()
		{
			@Override
			public void mouseUp(MouseEvent arg0)
			{
				ColorDialog dlg = new ColorDialog(parent.getShell());
				RGB rgb = dlg.open();
				if (rgb != null)
				{
					singleColorDisplay.setBackground(new Color(null, rgb.red, rgb.green,
							rgb.blue));
				}
			}
		});
		btnSelectColor.setText("change");

		btnRandomColors = new Button(grpInitialContent, SWT.RADIO);
		btnRandomColors.setText("Random colors");
		new Label(grpInitialContent, SWT.NONE);
		new Label(grpInitialContent, SWT.NONE);
	}

	/**
	 * Returns the number of colors the new palette should have
	 * 
	 * @return the number of colors for the new palette as an int
	 */
	protected int getColorDepth()
	{
		int i = Integer.parseInt(this.combo.getText());
		return i;
	}

	/**
	 * returns the selected red value
	 * 
	 * @return the red value (0..31)
	 */
	protected int getRedValue()
	{
		return (this.singleColorDisplay.getBackground().getRed() / 8);
	}

	/**
	 * returns the selected green value
	 * 
	 * @return the green value (0..31)
	 */
	protected int getGreenValue()
	{
		return (this.singleColorDisplay.getBackground().getGreen() / 8);
	}

	/**
	 * returns the selected blue value
	 * 
	 * @return the blue value (0..31)
	 */
	protected int getBlueValue()
	{
		return (this.singleColorDisplay.getBackground().getBlue() / 8);
	}

	protected String getImportMode()
	{
		if (btnSingleColor.getSelection())
		{
			return "single color";
		}
		return "random colors";
	}
}
