/**
 * 
 */
package de.digisalt.dsnesds.core.wizards.create.file.tileset;

import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.wb.swt.ResourceManager;

/**
 * @author flo
 */
public class NewTilesetWizardPage extends WizardPage
{
	private Spinner	spnColumnsOfTiles;
	private Spinner	spnRowsOfTiles;
	private Spinner	spnTotalNumberOfTiles;
	private Combo		cmbEncoding;
	private Spinner	spnLastRow;

	private boolean	reactOnRowSpinnerChanges			= true;
	private boolean	reactOnColumnSpinnerChanges		= true;
	private boolean	reactOnLastRowSpinnerChanges	= true;
	private boolean	reactOnTotalSpinnerChanges		= true;
	private Button	btnProjectColorPalette;
	private Combo		cmbProjectPalette;

	/**
	 * Create the wizard.
	 */
	public NewTilesetWizardPage()
	{
		super("wizardPage");
		setImageDescriptor(ResourceManager.getPluginImageDescriptor(
				"de.digisalt.dsnesds.core.wizards", "graphics/wizards/tileset.png"));
		setTitle("New tileset");
		setDescription("Create a new tileset with the following presets");
	}

	/**
	 * Create contents of the wizard.
	 * 
	 * @param parent
	 */
	@Override
	public void createControl(final Composite parent)
	{
		Composite container = new Composite(parent, SWT.NULL);

		setControl(container);
		container.setLayout(new GridLayout(2, false));

		Label lblEncoding = new Label(container, SWT.NONE);
		lblEncoding.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false,
				1, 1));
		lblEncoding.setText("Encoding");

		cmbEncoding = new Combo(container, SWT.READ_ONLY);
		cmbEncoding.setItems(new String[]
		{ "2 Colors (1bpp)", "4 Colors (2bpp)", "8 Colors (3bpp)",
				"16 Colors (4bpp)", "256 Colors (8bpp)", "Mode7 (8bpp)" });
		cmbEncoding.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false,
				1, 1));
		cmbEncoding.select(3);

		Group grpDimensions = new Group(container, SWT.NONE);
		grpDimensions.setLayout(new GridLayout(3, false));
		grpDimensions.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false,
				2, 1));
		grpDimensions.setText("Dimensions");

		Label lblHeightnumberOf = new Label(grpDimensions, SWT.NONE);
		lblHeightnumberOf.setText("Columns (Number of 8x8 Pixel Tiles)");

		this.spnColumnsOfTiles = new Spinner(grpDimensions, SWT.BORDER);
		spnColumnsOfTiles.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false,
				false, 1, 1));
		this.spnColumnsOfTiles.addModifyListener(new ModifyListener()
		{
			@Override
			public void modifyText(final ModifyEvent e)
			{
				if (reactOnRowSpinnerChanges)
				{
					reactOnRowSpinnerChanges = false;
					reactOnColumnSpinnerChanges = false;
					reactOnLastRowSpinnerChanges = false;
					reactOnTotalSpinnerChanges = false;

					// additional row mustn't be longer than number of columns
					if (NewTilesetWizardPage.this.spnLastRow != null
							&& NewTilesetWizardPage.this.spnColumnsOfTiles != null)
					{
						// maximum should be the number of columns
						NewTilesetWizardPage.this.spnLastRow
								.setMaximum(NewTilesetWizardPage.this.spnColumnsOfTiles
										.getSelection());
						// as well as the selected number
						if (NewTilesetWizardPage.this.spnLastRow.getSelection() > NewTilesetWizardPage.this.spnColumnsOfTiles
								.getSelection())
						{
							NewTilesetWizardPage.this.spnLastRow
									.setSelection(NewTilesetWizardPage.this.spnColumnsOfTiles
											.getSelection());
						}
					}

					reactOnRowSpinnerChanges = true;
					reactOnColumnSpinnerChanges = true;
					reactOnLastRowSpinnerChanges = true;
					reactOnTotalSpinnerChanges = true;

					// update Total Number Of Tiles
					updateTotalNumberOfTiles();
				}
			}
		});
		this.spnColumnsOfTiles.setMaximum(256);
		this.spnColumnsOfTiles.setMinimum(1);
		this.spnColumnsOfTiles.setSelection(8);
		new Label(grpDimensions, SWT.NONE);

		Label lblWidthnumberOf = new Label(grpDimensions, SWT.NONE);
		lblWidthnumberOf.setText("Rows (Number of 8x8 Pixel Tiles)");

		this.spnRowsOfTiles = new Spinner(grpDimensions, SWT.BORDER);
		spnRowsOfTiles.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false,
				false, 1, 1));
		this.spnRowsOfTiles.addModifyListener(new ModifyListener()
		{
			@Override
			public void modifyText(final ModifyEvent e)
			{
				if (reactOnColumnSpinnerChanges)
				{
					// update new total number of tiles
					NewTilesetWizardPage.this.updateTotalNumberOfTiles();
				}
			}
		});
		this.spnRowsOfTiles.setMaximum(256);
		this.spnRowsOfTiles.setMinimum(1);
		this.spnRowsOfTiles.setSelection(8);
		new Label(grpDimensions, SWT.NONE);

		Label lbladditionalIncomplete = new Label(grpDimensions, SWT.NONE);
		lbladditionalIncomplete.setText("+ (additional incomplete Row)");

		spnLastRow = new Spinner(grpDimensions, SWT.BORDER);
		spnLastRow.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false,
				1, 1));
		spnLastRow.addModifyListener(new ModifyListener()
		{
			@Override
			public void modifyText(ModifyEvent arg0)
			{
				if (reactOnLastRowSpinnerChanges)
				{
					// update total number of tiles
					updateTotalNumberOfTiles();
				}
			}
		});
		spnLastRow.setMaximum(8);
		new Label(grpDimensions, SWT.NONE);

		Label label = new Label(grpDimensions, SWT.SEPARATOR | SWT.HORIZONTAL);
		label.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 3, 1));

		Label lblNumberOfTiles = new Label(grpDimensions, SWT.NONE);
		lblNumberOfTiles.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false,
				false, 1, 1));
		lblNumberOfTiles.setText("Total number of Tiles");

		this.spnTotalNumberOfTiles = new Spinner(grpDimensions, SWT.BORDER);
		this.spnTotalNumberOfTiles.setSelection(64);
		this.spnTotalNumberOfTiles.addModifyListener(new ModifyListener()
		{
			@Override
			public void modifyText(final ModifyEvent e)
			{
				if (reactOnTotalSpinnerChanges)
				{
					// update Total Number Of Tiles
					updateInputFieldsOnTotalNumberChanged();
				}
			}
		});
		spnTotalNumberOfTiles.setMaximum(65536);
		this.spnTotalNumberOfTiles.setMinimum(0);

		Label lblNewLabel = new Label(grpDimensions, SWT.NONE);
		lblNewLabel.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false,
				1, 1));

		Group grpColorPalette = new Group(container, SWT.NONE);
		grpColorPalette.setLayout(new GridLayout(3, false));
		grpColorPalette.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, true,
				2, 1));
		grpColorPalette.setText("Preferred Color Palette");

		Button btnRandomColorPalette = new Button(grpColorPalette, SWT.RADIO);
		btnRandomColorPalette.setSelection(true);
		btnRandomColorPalette.setText("Random Color Palette");
		new Label(grpColorPalette, SWT.NONE);
		new Label(grpColorPalette, SWT.NONE);

		btnProjectColorPalette = new Button(grpColorPalette, SWT.RADIO);
		btnProjectColorPalette.setText("Project Color Palette");

		cmbProjectPalette = new Combo(grpColorPalette, SWT.READ_ONLY);
		GridData gd_combo_1 = new GridData(SWT.FILL, SWT.CENTER, true, false, 2, 1);
		gd_combo_1.widthHint = 358;
		cmbProjectPalette.setLayoutData(gd_combo_1);
	}

	private void updateTotalNumberOfTiles()
	{
		reactOnRowSpinnerChanges = false;
		reactOnColumnSpinnerChanges = false;
		reactOnLastRowSpinnerChanges = false;
		reactOnTotalSpinnerChanges = false;

		// recalculate total number of tiles
		if (NewTilesetWizardPage.this.spnTotalNumberOfTiles != null
				&& NewTilesetWizardPage.this.spnRowsOfTiles != null
				&& NewTilesetWizardPage.this.spnColumnsOfTiles != null
				&& NewTilesetWizardPage.this.spnLastRow != null)
		{
			NewTilesetWizardPage.this.spnTotalNumberOfTiles
					.setSelection(((NewTilesetWizardPage.this.spnRowsOfTiles
							.getSelection() * NewTilesetWizardPage.this.spnColumnsOfTiles
							.getSelection()) + NewTilesetWizardPage.this.spnLastRow
							.getSelection()));
		}

		reactOnRowSpinnerChanges = true;
		reactOnColumnSpinnerChanges = true;
		reactOnLastRowSpinnerChanges = true;
		reactOnTotalSpinnerChanges = true;
	}

	private void updateInputFieldsOnTotalNumberChanged()
	{
		reactOnRowSpinnerChanges = false;
		reactOnColumnSpinnerChanges = false;
		reactOnLastRowSpinnerChanges = false;
		reactOnTotalSpinnerChanges = false;

		// recalculate input numbers depending on output changed
		if (NewTilesetWizardPage.this.spnTotalNumberOfTiles != null
				&& NewTilesetWizardPage.this.spnRowsOfTiles != null
				&& NewTilesetWizardPage.this.spnColumnsOfTiles != null
				&& NewTilesetWizardPage.this.spnLastRow != null)
		{
			// do not change number of columns
			// nothing to be done here... :)
			// calculate number of rows
			NewTilesetWizardPage.this.spnRowsOfTiles.setSelection(new Double(Math
					.floor(NewTilesetWizardPage.this.spnTotalNumberOfTiles.getSelection()
							/ NewTilesetWizardPage.this.spnColumnsOfTiles.getSelection()))
					.intValue());
			// calculate additional row number
			NewTilesetWizardPage.this.spnLastRow
					.setSelection(NewTilesetWizardPage.this.spnTotalNumberOfTiles
							.getSelection()
							- (NewTilesetWizardPage.this.spnRowsOfTiles.getSelection() * NewTilesetWizardPage.this.spnColumnsOfTiles
									.getSelection()));
		}

		reactOnRowSpinnerChanges = true;
		reactOnColumnSpinnerChanges = true;
		reactOnLastRowSpinnerChanges = true;
		reactOnTotalSpinnerChanges = true;
	}

	// returns the bits to be used for one color
	protected int getEncoding()
	{
		String encoding = cmbEncoding.getText();
		switch (encoding)
		{
		case "2 Colors (1bpp)":
		{
			return 1;
		}
		case "4 Colors (2bpp)":
		{
			return 2;
		}
		case "8 Colors (3bpp)":
		{
			return 3;
		}
		case "16 Colors (4bpp)":
		{
			return 4;
		}
		default:
		{
			return 8;
		}
		}
	}

	// returns the number of colors/encoding mode
	protected String getEncodingMode()
	{
		return cmbEncoding.getText();
	}

	// returns the total number of tiles for the new tileset
	protected int getTotalNumberOfTiles()
	{
		return spnTotalNumberOfTiles.getSelection();
	}

	// number of columns to be used --> metadata to be attached to the file
	protected int getNumberOfColumns()
	{
		return spnColumnsOfTiles.getSelection();
	}

	// encoding to be used --> metadata to be attached to the file
	protected String getPreferredEncoding()
	{
		return cmbEncoding.getText();
	}

	// preferred palette to be used -> metadata to be attached to the file
	protected String getPreferredPalette()
	{
		String retStr = "";
		// project palette should be used --> return its name
		if (btnProjectColorPalette != null && cmbProjectPalette != null
				&& btnProjectColorPalette.getSelection())
		{
			return cmbProjectPalette.getText();
		}
		return retStr;
	}

}
