package de.digisalt.dsnesds.core.wizards.create.project.pages;

import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.wb.swt.ResourceManager;
import org.eclipse.wb.swt.SWTResourceManager;

import de.digisalt.dsnesds.core.model.mappings.projectmeta.CountrycodeMapping;

/**
 * @author flo The first page of the new project wizard for DSNESDS projects
 */
public class NewProjectPage1 extends WizardPage
{
	/**
	 * the form toolkit creating all UI resources
	 */
	private final FormToolkit	formToolkit	= new FormToolkit(Display.getDefault());
	/**
	 * the textfield containing the project name
	 */
	private Text							txtProjectName;
	/**
	 * the textfield containing the game longname
	 */
	private Text							txtGameName;
	/**
	 * the game shortname textfield
	 */
	private Text							txtGameShortname;
	private Button						btnCustomizeProjectData;
	private Spinner						spnVersion;
	private Group							grpProjectDetails;
	private Combo							cmbCarttype;
	private Combo							cmbCountrycode;
	private Combo							cmbRamsizes;
	private Combo							cmbRomsizes;
	private Combo							cmbLicensees;
	private Combo							cmbTiming;
	private Combo							cmbAdressing;
	private Combo							cmbVideoformat;

	/**
	 * Create the wizard.
	 */
	public NewProjectPage1()
	{
		super("wizardPage");
		setImageDescriptor(ResourceManager.getPluginImageDescriptor(
				"de.digisalt.dsnesds.core.wizards", "graphics/wizards/project.png"));
		setTitle("Create a DSNESDS Project");
		setDescription("Enter all relevant data to create a new SNES game project");
	}

	/**
	 * Create contents of the wizard.
	 * 
	 * @param parent
	 *          the wizard base component
	 */
	@Override
	public void createControl(final Composite parent)
	{
		Composite container = new Composite(parent, SWT.NULL);

		setControl(container);
		container.setLayout(new GridLayout(3, false));

		Label lblProjectName = this.formToolkit.createLabel(container,
				"Project name:", SWT.NONE);
		lblProjectName.setBackground(parent.getBackground());

		this.txtProjectName = this.formToolkit.createText(container, "New Text",
				SWT.NONE);
		this.txtProjectName.setMessage("Enter the project name here");
		this.txtProjectName.addModifyListener(new ModifyListener()
		{
			@Override
			public void modifyText(final ModifyEvent e)
			{
				updateEnablement();
				updateGamenameFromProjectNameChange();
			}
		});
		this.txtProjectName.setText("");
		GridData gd_txtProjectName = new GridData(SWT.FILL, SWT.CENTER, true,
				false, 2, 1);
		gd_txtProjectName.widthHint = 401;
		this.txtProjectName.setLayoutData(gd_txtProjectName);

		Label lblGameName = this.formToolkit.createLabel(container, "Game Name:",
				SWT.NONE);
		lblGameName.setBackground(parent.getBackground());

		this.txtGameName = this.formToolkit.createText(container, "New Text",
				SWT.NONE);
		this.txtGameName.setTextLimit(21);
		this.txtGameName.setMessage("Enter the games name here");
		this.txtGameName.addModifyListener(new ModifyListener()
		{
			@Override
			public void modifyText(final ModifyEvent e)
			{
				updateEnablement();
				updateGamenameFromGamenameChange();
			}
		});
		this.txtGameName.setText("");
		this.txtGameName.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true,
				false, 2, 1));

		Button btnUseDefault = new Button(container, SWT.RADIO);
		btnUseDefault.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(final SelectionEvent e)
			{
				disableDetailSelection();
			}
		});
		btnUseDefault.setSelection(true);
		this.formToolkit.adapt(btnUseDefault, true, true);
		btnUseDefault.setText("use project defaults");
		btnUseDefault.setBackground(parent.getBackground());
		new Label(container, SWT.NONE);

		this.btnCustomizeProjectData = new Button(container, SWT.RADIO);
		this.btnCustomizeProjectData.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(final SelectionEvent e)
			{
				enableDetailSelection();
			}
		});
		this.formToolkit.adapt(this.btnCustomizeProjectData, true, true);
		this.btnCustomizeProjectData.setText("customize project data");
		this.btnCustomizeProjectData.setBackground(parent.getBackground());

		this.grpProjectDetails = new Group(container, SWT.NONE);
		this.grpProjectDetails.setEnabled(false);
		this.grpProjectDetails.setLayout(new GridLayout(3, false));
		this.grpProjectDetails.setLayoutData(new GridData(SWT.FILL, SWT.FILL,
				false, false, 3, 1));
		this.grpProjectDetails.setText("Project details");
		this.grpProjectDetails.setBackground(parent.getBackground());
		this.formToolkit.adapt(this.grpProjectDetails);
		this.formToolkit.paintBordersFor(this.grpProjectDetails);
		this.grpProjectDetails.setBackground(parent.getBackground());
		this.grpProjectDetails.setForeground(SWTResourceManager
				.getColor(SWT.COLOR_WIDGET_FOREGROUND));

		Label lblGameShortname = this.formToolkit.createLabel(
				this.grpProjectDetails, "Game Shortname:", SWT.NONE);
		lblGameShortname.setEnabled(false);
		lblGameShortname.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false,
				false, 2, 1));
		lblGameShortname.setBackground(parent.getBackground());

		this.txtGameShortname = this.formToolkit.createText(this.grpProjectDetails,
				"New Text", SWT.NONE);
		this.txtGameShortname.setMessage("Enter the games shortname here");
		this.txtGameShortname.setTextLimit(4);
		this.txtGameShortname.setEnabled(false);
		GridData gd_txtGameShortname = new GridData(SWT.FILL, SWT.CENTER, true,
				false, 1, 1);
		gd_txtGameShortname.widthHint = 280;
		this.txtGameShortname.setLayoutData(gd_txtGameShortname);
		this.txtGameShortname.setText("");

		Label lblRomTiming = this.formToolkit.createLabel(this.grpProjectDetails,
				"Rom Timing:", SWT.NONE);
		lblRomTiming.setEnabled(false);
		lblRomTiming.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, false,
				2, 1));
		lblRomTiming.setBackground(parent.getBackground());

		this.cmbTiming = new Combo(this.grpProjectDetails, SWT.READ_ONLY);
		this.cmbTiming.setItems(new String[]
		{ "120 Nanoseconds (FASTROM)", "200 Nanoseconds (SLOWROM)" });
		this.cmbTiming.setEnabled(false);
		this.cmbTiming.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true,
				false, 1, 1));
		this.formToolkit.adapt(this.cmbTiming);
		this.formToolkit.paintBordersFor(this.cmbTiming);
		this.cmbTiming.select(0);

		Label lblAdressing = this.formToolkit.createLabel(this.grpProjectDetails,
				"Adressing:", SWT.NONE);
		lblAdressing.setEnabled(false);
		lblAdressing.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, false,
				2, 1));
		lblAdressing.setBackground(parent.getBackground());

		this.cmbAdressing = new Combo(this.grpProjectDetails, SWT.READ_ONLY);
		this.cmbAdressing.setItems(new String[]
		{ "LoROM", "HiROM" });
		this.cmbAdressing.setEnabled(false);
		this.cmbAdressing.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true,
				false, 1, 1));
		this.formToolkit.adapt(this.cmbAdressing);
		this.formToolkit.paintBordersFor(this.cmbAdressing);
		this.cmbAdressing.select(1);

		Label lblCartridgeType = this.formToolkit.createLabel(
				this.grpProjectDetails, "Cartridge Type:", SWT.NONE);
		lblCartridgeType.setEnabled(false);
		lblCartridgeType.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false,
				false, 2, 1));
		lblCartridgeType.setBackground(parent.getBackground());

		this.cmbCarttype = new Combo(this.grpProjectDetails, SWT.READ_ONLY);
		this.cmbCarttype.addMouseListener(new MouseAdapter()
		{
			@Override
			public void mouseUp(final MouseEvent e)
			{
				if (NewProjectPage1.this.cmbCarttype.getSelectionIndex() == 0
						|| NewProjectPage1.this.cmbCarttype.getSelectionIndex() == 3
						|| NewProjectPage1.this.cmbCarttype.getSelectionIndex() == 6)
				{
					NewProjectPage1.this.cmbRamsizes.select(0);
				}
			}
		});
		this.cmbCarttype.setEnabled(false);
		this.cmbCarttype.setItems(new String[]
		{ "ROM", "ROM + RAM", "ROM + SRAM", "ROM + DSP", "ROM + RAM + DSP",
				"ROM + SRAM + DSP", "ROM + SuperFX", "ROM + RAM + SuperFX",
				"ROM + SRAM + SuperFX", "ROM + SRAM + SA-1" });
		this.cmbCarttype.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true,
				false, 1, 1));
		this.formToolkit.adapt(this.cmbCarttype);
		this.formToolkit.paintBordersFor(this.cmbCarttype);
		this.cmbCarttype.select(2);

		Label lblRomSize = this.formToolkit.createLabel(this.grpProjectDetails,
				"ROM Size:", SWT.NONE);
		lblRomSize.setEnabled(false);
		lblRomSize.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, false,
				2, 1));
		lblRomSize.setBackground(parent.getBackground());

		this.cmbRomsizes = new Combo(this.grpProjectDetails, SWT.READ_ONLY);
		this.cmbRomsizes.setItems(new String[]
		{ "256 Kilobytes", "512 Kilobytes", "1 Megabyte", "2 Megabytes",
				"4 Megabytes" });
		this.cmbRomsizes.setEnabled(false);
		this.cmbRomsizes.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true,
				false, 1, 1));
		this.formToolkit.adapt(this.cmbRomsizes);
		this.formToolkit.paintBordersFor(this.cmbRomsizes);
		this.cmbRomsizes.select(4);

		Label lblRamSize = this.formToolkit.createLabel(this.grpProjectDetails,
				"RAM Size:", SWT.NONE);
		lblRamSize.setEnabled(false);
		lblRamSize.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, false,
				2, 1));
		lblRamSize.setBackground(parent.getBackground());

		this.cmbRamsizes = new Combo(this.grpProjectDetails, SWT.READ_ONLY);
		this.cmbRamsizes.setEnabled(false);
		this.cmbRamsizes.setItems(new String[]
		{ "0 Kilobytes", "2 Kilobytes", "4 Kilobytes", "8 Kilobytes" });
		this.cmbRamsizes.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true,
				false, 1, 1));
		this.formToolkit.adapt(this.cmbRamsizes);
		this.formToolkit.paintBordersFor(this.cmbRamsizes);
		this.cmbRamsizes.select(3);

		Label lblCountrycode = this.formToolkit.createLabel(this.grpProjectDetails,
				"Countrycode:", SWT.NONE);
		lblCountrycode.setEnabled(false);
		lblCountrycode.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false,
				false, 2, 1));
		lblCountrycode.setBackground(parent.getBackground());

		this.cmbCountrycode = new Combo(this.grpProjectDetails, SWT.READ_ONLY);
		this.cmbCountrycode.setEnabled(false);
		this.cmbCountrycode.setItems(new CountrycodeMapping().getLabelSet());
		this.cmbCountrycode.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true,
				false, 1, 1));
		this.formToolkit.adapt(this.cmbCountrycode);
		this.formToolkit.paintBordersFor(this.cmbCountrycode);
		this.cmbCountrycode.select(11);

		Label lblLicensee = this.formToolkit.createLabel(this.grpProjectDetails,
				"Licensee:", SWT.NONE);
		lblLicensee.setEnabled(false);
		lblLicensee.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, false,
				2, 1));
		lblLicensee.setBackground(parent.getBackground());

		this.cmbLicensees = new Combo(this.grpProjectDetails, SWT.READ_ONLY);
		this.cmbLicensees.setEnabled(false);
		this.cmbLicensees.setItems(new String[]
		{ "Absolute Entertainment", "Acclaim", "Accolade Europe Inc.",
				"Activision", "Ajinomoto", "American Sammy",
				"American Softworks Corp.", "Arcade Zone", "ASCII", "Ask Kodansha",
				"Asmik", "Athena", "Atlus", "Banalex", "BanDai America", "Banpresto",
				"Beam Software", "BEC", "Black Pearl", "Bullet-Proof Software",
				"Capcom", "Carrozzeria", "Character Soft",
				"Chris Gray Enterprises Inc.", "Chun Software", "COBRA Team",
				"Coconuts", "Culture Brain", "Data East", "DATAM-Polystar", "Digisalt",
				"Dynamic", "Electro Brain", "Electronic Arts", "Elite",
				"Empire Software", "Enix", "Epoch", "Falcom", "Field", "Game Village",
				"GameTek", "Gremlin Graphics", "Halken", "Hect", "Hi Tech Expressions",
				"Hiro", "Hori", "HOT-B", "Hudson Soft", "Human Entertainment", "I'Max",
				"Imagesoft", "Imagineer", "Infogrames", "INTEC Inc.", "Interplay",
				"Irem Japan", "Jaleco", "JVC", "K. Amusement", "Kaneco", "Kemco",
				"King A Wave", "KOEI", "Konami", "KSS", "Laser Beam",
				"LifeFitness Exertainment", "LJN Toys", "Loriciel", "Lozc",
				"LucasArts", "Magifact", "Masaya", "Maxis", "Mebio Software",
				"Meldac KAZe", "Micro World", "Micronet", "Microprose", "Mindscape",
				"Misawa", "Namco Ltd.", "Natsume", "Naxat", "NCS", "NHK", "Nichibutsu",
				"Nintendo", "Ocean", "Open System", "Origin", "Other", "Pack in Video",
				"Pal Soft", "Parker Brothers", "Pony Canyon", "POW", "Psygnosis Ltd.",
				"Quest", "Rage Software", "Renovation Products", "Ringler Studios",
				"Romstar", "S'Pal Inc.", "Sculptured Software", "Seika Corp.",
				"Seta Co. Ltd.", "Sigma", "Sofel", "Sony", "Sotsu Agency",
				"Spectrum Holobyte", "Squaresoft", "Starfish", "STORM", "Sunsoft",
				"System 3", "System Sacom Corp.", "T&E Software", "Taito", "Takara",
				"Tec Magik", "Technos", "Tecmo", "Teichio", "THQ Software", "Titus",
				"Toei Animation", "Toho", "Tokai Engineering",
				"Tokuma Shoten Intermedia", "Tomy", "Tonkin House",
				"Toshiba EMI System Vision", "Tradewest", "Triffix Entertainment",
				"Tsukuda Original", "U.S. Gold", "UBI Soft", "Varie",
				"Viacom New Media", "Video System", "Virgin Interactive Entertainment",
				"Visit Co. Ltd.", "Wolf Team", "Yanoman", "Zamuse", "Zoom" });
		this.cmbLicensees.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true,
				false, 1, 1));
		this.formToolkit.adapt(this.cmbLicensees);
		this.formToolkit.paintBordersFor(this.cmbLicensees);
		this.cmbLicensees.select(93);

		Label lblVideoFormat = this.formToolkit.createLabel(this.grpProjectDetails,
				"Video Format:", SWT.NONE);
		lblVideoFormat.setEnabled(false);
		lblVideoFormat.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false,
				false, 2, 1));
		lblVideoFormat.setBackground(parent.getBackground());

		this.cmbVideoformat = new Combo(this.grpProjectDetails, SWT.READ_ONLY);
		this.cmbVideoformat.setEnabled(false);
		this.cmbVideoformat.setItems(new String[]
		{ "NTSC", "PAL" });
		this.cmbVideoformat.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true,
				false, 1, 1));
		this.formToolkit.adapt(this.cmbVideoformat);
		this.formToolkit.paintBordersFor(this.cmbVideoformat);
		this.cmbVideoformat.select(0);

		Label lblVersion = this.formToolkit.createLabel(this.grpProjectDetails,
				"Version:", SWT.NONE);
		lblVersion.setEnabled(false);
		lblVersion.setBackground(parent.getBackground());

		Label label = new Label(this.grpProjectDetails, SWT.NONE);
		label.setEnabled(false);
		label
				.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
		this.formToolkit.adapt(label, true, true);
		label.setText("1.");
		label.setBackground(parent.getBackground());

		this.spnVersion = new Spinner(this.grpProjectDetails, SWT.BORDER);
		this.spnVersion.setEnabled(false);
		this.spnVersion.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false,
				false, 1, 1));
		this.spnVersion.setMaximum(255);
		this.formToolkit.adapt(this.spnVersion);
		this.formToolkit.paintBordersFor(this.spnVersion);
		new Label(this.grpProjectDetails, SWT.NONE);
		new Label(this.grpProjectDetails, SWT.NONE);

		Button btnResetToDefault = new Button(this.grpProjectDetails, SWT.NONE);
		btnResetToDefault.addKeyListener(new KeyAdapter()
		{
			@Override
			public void keyReleased(final KeyEvent e)
			{
				if (e.keyCode == SWT.SPACE || e.keyCode == SWT.CR)
				{
					resetDetails();
				}
			}
		});
		btnResetToDefault.addMouseListener(new MouseAdapter()
		{
			@Override
			public void mouseUp(final MouseEvent e)
			{
				resetDetails();
			}
		});
		btnResetToDefault.setEnabled(false);
		btnResetToDefault.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false,
				false, 1, 1));
		this.formToolkit.adapt(btnResetToDefault, true, true);
		btnResetToDefault.setText("Reset to default values");

		updateEnablement();
	}

	/**
	 * will update the fields gamename and gameshortname according to the recent
	 * changes in project field
	 */
	protected void updateGamenameFromProjectNameChange()
	{
		String projectName = this.txtProjectName.getText();
		// update Game name
		if (this.txtGameName != null)
		{
			if (projectName.length() > 21)
			{
				this.txtGameName.setText(projectName.substring(0, 21).toUpperCase());
			}
			else
			{
				this.txtGameName.setText(projectName.toUpperCase());
			}
		}
		// update game shortname
		if (this.txtGameShortname != null)
		{
			if (projectName.length() > 4)
			{
				this.txtGameShortname
						.setText(projectName.substring(0, 4).toUpperCase());
			}
			else
			{
				this.txtGameShortname.setText(projectName.toUpperCase());
			}
		}
	}

	/**
	 * will update the field gameshortname according to the recent changes in
	 * gamename field
	 */
	protected void updateGamenameFromGamenameChange()
	{
		// update game shortname
		if (this.txtGameShortname != null)
		{
			String gameName = this.txtGameName.getText();
			if (gameName.length() > 4)
			{
				this.txtGameShortname.setText(gameName.substring(0, 4).toUpperCase());
			}
			else
			{
				this.txtGameShortname.setText(gameName.toUpperCase());
			}
		}
	}

	/**
	 * resets all details fields to the default values
	 */
	protected void resetDetails()
	{
		if (this.txtGameName.getText().length() > 0)
		{
			if (this.txtGameName.getText().length() >= 4)
			{
				this.txtGameShortname.setText(this.txtGameName.getText()
						.substring(0, 4));
			}
			else
			{
				this.txtGameShortname.setText(this.txtGameName.getText());
			}
		}
		else
		{
			if (this.txtProjectName.getText().length() > 0)
			{
				if (this.txtProjectName.getText().length() >= 4)
				{
					this.txtGameShortname.setText(this.txtProjectName.getText()
							.substring(0, 4));
				}
				else
				{
					this.txtGameShortname.setText(this.txtProjectName.getText());
				}
			}
			else
			{
				this.txtGameShortname.setText("");
			}
		}
		NewProjectPage1.this.cmbTiming.select(0);
		NewProjectPage1.this.cmbAdressing.select(1);
		NewProjectPage1.this.cmbCarttype.select(2);
		NewProjectPage1.this.cmbRomsizes.select(4);
		NewProjectPage1.this.cmbRamsizes.select(3);
		NewProjectPage1.this.cmbCountrycode.select(11);
		NewProjectPage1.this.cmbLicensees.select(93);
		NewProjectPage1.this.cmbVideoformat.select(0);
		NewProjectPage1.this.spnVersion.setDigits(0);
	}

	/**
	 * A method returning the chosen project name
	 * 
	 * @return the project name chosen
	 */
	public String getProjectName()
	{
		return this.txtProjectName.getText();
	}

	/**
	 * A method returning the chosen game name
	 * 
	 * @return the game name chosen
	 */
	public String getGameName()
	{
		return this.txtGameName.getText();
	}

	/**
	 * activates/deactivates the next and finish buttons depending on weather or
	 * not a project has been selected
	 */
	private void updateEnablement()
	{
		if (this.txtProjectName != null && this.txtGameName != null)
		{
			boolean projectNameSet = this.txtProjectName.getText() != "";
			boolean gameNameSet = this.txtGameName.getText() != "";

			if (projectNameSet && gameNameSet)
			{
				setErrorMessage(null);
				setDescription("Click Finish to create the new Project.");
				setPageComplete(true);
			}
			else
			{
				if (!projectNameSet)
				{
					setErrorMessage("You have to enter a project name!");
				}
				else if (!gameNameSet)
				{
					setErrorMessage("You have to enter the games name!");
				}
				setPageComplete(false);
			}
		}
		else
		{
			setPageComplete(false);
		}
	}

	/**
	 * this method will enable all detail selection widgets of the page
	 */
	private void enableDetailSelection()
	{
		changeEnablementOfDetails(true);
	}

	/**
	 * this method will disable all detail selection widgets of the page
	 */
	private void disableDetailSelection()
	{
		changeEnablementOfDetails(false);
	}

	/**
	 * This method will enable or disable the detail selection widgets of the page
	 * depending on the boolean parameter
	 * 
	 * @param enableDetails
	 *          if true, widgets will be enabled, disabled otherwise
	 */
	private void changeEnablementOfDetails(final boolean enableDetails)
	{
		this.grpProjectDetails.setEnabled(enableDetails);
		for (Control c : this.grpProjectDetails.getChildren())
		{
			c.setEnabled(enableDetails);
		}
	}

	/**
	 * returns the game shortname
	 * 
	 * @return the game shortname
	 */
	public String getShortname()
	{
		return this.txtGameShortname.getText();
	}

	/**
	 * returns the timing currently set
	 * 
	 * @return the timing set
	 */
	public String getTiming()
	{
		return this.cmbTiming.getText();
	}

	/**
	 * returns the currently set adressing mode
	 * 
	 * @return the adressing mode set
	 */
	public String getAdressing()
	{
		return this.cmbAdressing.getText();
	}

	/**
	 * returns the currently set cartridge type
	 * 
	 * @return the carttype set
	 */
	public String getCarttype()
	{
		return this.cmbCarttype.getText();
	}

	/**
	 * returns the currently set romsize
	 * 
	 * @return the romsize set
	 */
	public String getRomsize()
	{
		return this.cmbRomsizes.getText();
	}

	/**
	 * returns the ramsize set
	 * 
	 * @return the ramsize set
	 */
	public String getRamsize()
	{
		return this.cmbRamsizes.getText();
	}

	/**
	 * returns the country currently selected
	 * 
	 * @return the country selected
	 */
	public String getCountry()
	{
		return this.cmbCountrycode.getText();
	}

	/**
	 * returns the licensee currently selected
	 * 
	 * @return the licensee set
	 */
	public String getLicensee()
	{
		return this.cmbLicensees.getText();
	}

	/**
	 * returns the videoformat to be used
	 * 
	 * @return the videoformat (PAL/NTSC)
	 */
	public String getVideoFormat()
	{
		return this.cmbVideoformat.getText();
	}

	/**
	 * returns the version to be used
	 * 
	 * @return the version
	 */
	public int getVersion()
	{
		return this.spnVersion.getSelection();
	}
}
