package de.digisalt.dsnesds.core.project;

import java.util.Map;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResourceDelta;
import org.eclipse.core.resources.IResourceDeltaVisitor;
import org.eclipse.core.resources.IncrementalProjectBuilder;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;

import de.digisalt.dsnesds.core.model.objects.DsnesdsProjectRoot;
import de.digisalt.dsnesds.core.model.util.ProjectModelUtil;

/**
 * @author flo
 * 
 *         The DSNESDS project builder... will load all resource content into
 *         ram for reference
 */
public class DsnesdsProjectBuilder extends IncrementalProjectBuilder
{
	/**
	 * the ID for reference in plugin.xml
	 */
	public static final String id = "de.digisalt.dsnesds.core.project.DsnesdsProjectBuilder";
	
	@Override
	protected IProject[] build(final int kind,
			@SuppressWarnings("rawtypes") final Map args,
			final IProgressMonitor monitor) throws CoreException
	{
		if (kind == IncrementalProjectBuilder.FULL_BUILD)
		{
			fullBuild(monitor);
		} else
		{
			IResourceDelta delta = getDelta(getProject());
			if (delta == null)
			{
				fullBuild(monitor);
			} else
			{
				incrementalBuild(delta, monitor);
			}
		}
		return null;
	}
	
	/**
	 * This will incrementally build everything needed as the delta has been
	 * changed
	 * 
	 * @param delta
	 *          the resource delta to react upon
	 * @param monitor
	 *          the progress monitor to use for job progress indication
	 */
	private void incrementalBuild(final IResourceDelta delta,
			final IProgressMonitor monitor)
	{
		System.out.println("incremental build on " + delta);
		// TODO: implement
		// Reference:
		// http://wiki.eclipse.org/FAQ_How_do_I_implement_an_incremental_project_builder%3F
		try
		{
			delta.accept(new IResourceDeltaVisitor()
			{
				@Override
				public boolean visit(final IResourceDelta delta)
				{
					System.out
							.println("changed: " + delta.getResource().getRawLocation());
					return true; // visit children too
				}
			});
		} catch (CoreException e)
		{
			e.printStackTrace();
		}
	}
	
	/**
	 * This will be invoked if all workingdata should be removed and recreated
	 * 
	 * @param monitor
	 *          the job monitor to be used
	 * @throws CoreException
	 *           may occur if there was a problem reading the project resources
	 */
	private void fullBuild(final IProgressMonitor monitor) throws CoreException
	{
		System.out.println("full build");
		DsnesdsProjectRoot project = ProjectModelUtil
				.recreateProjectRoot(getProject());
		// TODO: implement
		// Reference:
		// http://wiki.eclipse.org/FAQ_How_do_I_implement_an_incremental_project_builder%3F
	}
}
