/**
 * 
 */
package de.digisalt.dsnesds.digui;

import java.util.HashSet;

import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Device;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Composite;

/**
 * @author flo
 */
public abstract class DigUiComposite extends Canvas
{
	/**
	 * a vector containing all SWT colors (they's all be disposed at the end of
	 * the widgets lifecycle automatically
	 */
	private final HashSet<Color>	colors	= new HashSet<Color>();
	private final HashSet<Font>		fonts		= new HashSet<Font>();

	/**
	 * @param parent
	 *          the parent composite to add this one to
	 * @param style
	 *          the SWT style constant
	 */
	public DigUiComposite(final Composite parent, final int style)
	{
		super(parent, style);

		addDisposeListener(new DisposeListener()
		{
			/**
			 * Dispose all colors used within this widget
			 */
			@Override
			public void widgetDisposed(final DisposeEvent e)
			{
				for (Color c : DigUiComposite.this.colors)
				{
					c.dispose();
				}
				for (Font f : DigUiComposite.this.fonts)
				{
					f.dispose();
				}
			}
		});

		addPaintListener(new PaintListener()
		{
			@Override
			public void paintControl(final PaintEvent e)
			{
				Point p = getSize();
				int width = p.x;
				int height = p.y;

				drawContent(e, e.gc, width, height);
			}

		});
	}

	/**
	 * This method is used to create new SWT colors for this widget. They're
	 * disposed automatically once the widget is disposed.
	 * 
	 * @param r
	 *          the red value (0-255)
	 * @param g
	 *          the green value (0-255)
	 * @param b
	 *          the blue value (0-255)
	 * @return the SWT color object created
	 */
	public Color createColor(final int r, final int g, final int b)
	{
		Color c = new Color(null, r, g, b);
		if (!this.colors.contains(c))
		{
			this.colors.add(c);
		}
		return c;
	}

	public Font createFont(final Device device, final String name,
			final int height, final int style)
	{
		Font f = new Font(device, name, height, style);
		if (!this.fonts.contains(f))
		{
			this.fonts.add(f);
		}
		return f;
	}

	public abstract void drawContent(final PaintEvent e, GC gc, int width,
			int height);
}
