/**
 * 
 */
package de.digisalt.dsnesds.digui;

import java.util.Random;
import java.util.Vector;

import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.widgets.Composite;

/**
 * @author flo
 */
public class PalettePanel extends DigUiComposite
{

	/**
	 * Temporarily used to display colors even though no input was set
	 */
	private final Vector<Color>	colorss							= new Vector<Color>();

	/**
	 * The internal selection number for the left mouse button
	 */
	private int									leftSelectionIndex	= 0;
	/**
	 * The internal selection number for the right mouse button
	 */
	private int									rightSelectionIndex	= 1;

	/**
	 * The container for the model. May be of type Palette or PaletteObject
	 */
	Object											pal									= null;

	/**
	 * Will temporarily store if the last click on a color was left or right click
	 */
	private boolean							lastClickWasLeft		= true;

	/**
	 * The width of one color blob
	 */
	private int									blobwidth;

	/**
	 * the height of one color blob
	 */
	private int									blobheight;

	/**
	 * @param parent
	 *          the parent composite to add this one to
	 * @param style
	 *          the style constant to use
	 */
	public PalettePanel(final Composite parent, final int style)
	{
		super(parent, style);

		Random randGen = new Random(System.currentTimeMillis());
		for (int i = 0; i < 256; i++)
		{
			int r = Math.abs(randGen.nextInt()) % 255;
			int g = Math.abs(randGen.nextInt()) % 255;
			int b = Math.abs(randGen.nextInt()) % 255;

			this.colorss.add(createColor(r, g, b));
		}

		this.addMouseListener(new MouseListener()
		{
			@Override
			public void mouseDoubleClick(final MouseEvent e)
			{
			}

			@Override
			public void mouseDown(final MouseEvent e)
			{
			}

			@Override
			public void mouseUp(final MouseEvent e)
			{
				int i = getClickedColorIndex(e.x, e.y);
				if (e.button == 1)
				{
					setLeftSelectionIndex(i);
				}
				else if (e.button == 3)
				{
					setRightSelectionIndex(i);
				}
			}

		});
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * de.digisalt.dsnesds.digui.DigUiComposite#drawContent(org.eclipse.swt.events
	 * .PaintEvent)
	 */
	@Override
	public void drawContent(final PaintEvent e, final GC gc, final int width,
			final int height)
	{
		if (this.colorss.size() == 256)
		{
			this.blobwidth = (int) Math.floor(((width / 26) - 1));
			this.blobheight = (int) Math.floor(((height / 10) - 1));

			// current index of the color arrays
			int i = 0;

			// draw all palette colors
			for (int row = 0; row < 10; row++)
			{
				for (int col = 0; col < 26; col++)
				{
					// handle last four color places in last row --> leave empty
					if (i >= 256)
					{
						// do nothing... don't paint these squares
					}
					else
					{
						// draw color
						gc.setBackground(this.colorss.get(i));
						gc.fillRectangle(1 + (col * (this.blobwidth + 1)),
								1 + (row * (this.blobheight + 1)), this.blobwidth,
								this.blobheight);
						// draw selection frame if necessary
						if (i == this.leftSelectionIndex || i == this.rightSelectionIndex)
						{
							gc.setForeground(createColor(0, 0, 0));
							gc.drawRectangle(1 + (col * (this.blobwidth + 1)),
									1 + (row * (this.blobheight + 1)), this.blobwidth,
									this.blobheight);
						}
					}
					i++;
				}
			}
		}
		else if (this.colorss.size() == 16)
		{
			// draw the 16 color palette
			// calculate width and height of each color blob
			this.blobwidth = (int) Math.floor(((width / 8) - 3));
			this.blobheight = (int) Math.floor(((height / 2) - 3));

			// current index of the color arrays
			int i = 0;

			// draw all palette colors
			for (int row = 0; row < 2; row++)
			{
				for (int col = 0; col < 8; col++)
				{
					// handle last four color places in last row --> leave empty
					gc.setBackground(this.colorss.get(i));
					gc.fillRectangle(1 + (col * (this.blobwidth + 3)),
							1 + (row * (this.blobheight + 3)), this.blobwidth,
							this.blobheight);
					// draw selection border if necessary
					if (i == this.leftSelectionIndex || i == this.rightSelectionIndex)
					{
						gc.setForeground(createColor(0, 0, 0));
						gc.drawRectangle(1 + (col * (this.blobwidth + 3)),
								1 + (row * (this.blobheight + 3)), this.blobwidth,
								this.blobheight);
					}
					i++;
				}

			}
		}
		else if (this.colorss.size() == 8)
		{
			// draw the 8 color palette
			// calculate width and height of each color blob
			this.blobwidth = (int) Math.floor(((width / 8) - 3));
			this.blobheight = (int) Math.floor(((height) - 3));

			// current index of the color arrays
			int i = 0;

			// draw all palette colors
			for (int col = 0; col < 8; col++)
			{
				// handle last four color places in last row --> leave empty
				gc.setBackground(this.colorss.get(i));
				gc.fillRectangle(1 + (col * (this.blobwidth + 3)), 1, this.blobwidth,
						this.blobheight);
				// draw selection highlighter if necessary
				if (i == this.leftSelectionIndex || i == this.rightSelectionIndex)
				{
					gc.setForeground(createColor(0, 0, 0));
					gc.drawRectangle(1 + (col * (this.blobwidth + 3)), 1, this.blobwidth,
							this.blobheight);
				}
				i++;
			}
		}
		else if (this.colorss.size() == 4)
		{
			// draw the 4 color palette
			// calculate width and height of each color blob
			this.blobwidth = (int) Math.floor(((width / 4) - 3));
			this.blobheight = (int) Math.floor(((height) - 3));

			// current index of the color arrays
			int i = 0;

			// draw all palette colors
			for (int col = 0; col < 4; col++)
			{
				// handle last four color places in last row --> leave empty
				gc.setBackground(this.colorss.get(i));
				gc.fillRectangle(1 + (col * (this.blobwidth + 3)), 1, this.blobwidth,
						this.blobheight);
				// draw selection highlighter if necessary
				if (i == this.leftSelectionIndex || i == this.rightSelectionIndex)
				{
					gc.setForeground(createColor(0, 0, 0));
					gc.drawRectangle(1 + (col * (this.blobwidth + 3)), 1, this.blobwidth,
							this.blobheight);
				}
				i++;
			}
		}
		else if (this.colorss.size() == 2)
		{
			// draw the 2 color palette
			// calculate width and height of each color blob
			this.blobwidth = (int) Math.floor(((width / 2) - 3));
			this.blobheight = (int) Math.floor(((height) - 3));

			// current index of the color arrays
			int i = 0;

			// draw all palette colors
			for (int col = 0; col < 2; col++)
			{
				// handle last four color places in last row --> leave empty
				gc.setBackground(this.colorss.get(i));
				gc.fillRectangle(1 + (col * (this.blobwidth + 3)), 1, this.blobwidth,
						this.blobheight);
				// draw selection highlighter if necessary
				if (i == this.leftSelectionIndex || i == this.rightSelectionIndex)
				{
					gc.setForeground(createColor(0, 0, 0));
					gc.drawRectangle(1 + (col * (this.blobwidth + 3)), 1, this.blobwidth,
							this.blobheight);
				}
				i++;
			}
		}
		else
		{
			Color red = createColor(255, 0, 0);
			gc.setForeground(red);
			gc.drawString("Test", 1, 1);

			gc.drawRectangle(10, 10, 20, 20);
		}
	}

	/**
	 * returns the index of the color that is at pixel x/y
	 * 
	 * @param x
	 *          (x coordinate of the clicked event)
	 * @param y
	 *          (y coordinate of the clicked event)
	 * @return i (the index number of the clicked color
	 */
	private int getClickedColorIndex(final int x, final int y)
	{
		// depending on the number of colors, the colors have different sensitive
		// areass
		if (this.colorss.size() == 256)
		{
			// scale factor one: 7x7 blobs
			int col = ((Double) (Math.ceil(((double) x / (double) this.blobwidth))))
					.intValue() - 1;
			int row = ((Double) (Math.ceil(((double) y / (double) this.blobheight))))
					.intValue();
			if (col < 0)
			{
				col = 0;
			}
			if (col >= 26)
			{
				col = 25;
			}
			if (row < 0)
			{
				row = 0;
			}
			if (row >= 11)
			{
				row = 10;
			}
			return ((26 * (row - 1)) + col);
		}
		else if (this.colorss.size() == 16)
		{
			// 23x36
			int col = ((Double) (Math.ceil(((double) x / (double) 23)))).intValue() - 1;
			int row = ((Double) (Math.ceil(((double) y / (double) 36)))).intValue();
			if (col < 0)
			{
				col = 0;
			}
			if (col >= 8)
			{
				col = 7;
			}
			if (row < 0)
			{
				row = 0;
			}
			if (row >= 3)
			{
				row = 2;
			}
			return ((8 * (row - 1)) + col);
		}
		else if (this.colorss.size() == 8)
		{
			// 23
			int col = ((Double) (Math.ceil(((double) x / (double) 23)))).intValue() - 1;
			if (col < 0)
			{
				col = 0;
			}
			if (col >= 8)
			{
				col = 7;
			}
			return col;
		}
		else if (this.colorss.size() == 4)
		{
			// 46
			int col = ((Double) (Math.ceil(((double) x / (double) 46)))).intValue() - 1;
			if (col < 0)
			{
				col = 0;
			}
			if (col >= 4)
			{
				col = 3;
			}
			return col;
		}
		else
		// if (numberOfColors == 2)
		{
			// 92
			int col = ((Double) (Math.ceil(((double) x / (double) 92)))).intValue() - 1;
			if (col < 0)
			{
				col = 0;
			}
			if (col >= 2)
			{
				col = 1;
			}
			return col;
		}
	}

	/**
	 * @return the rightSelectionIndex
	 */
	public int getRightSelectionIndex()
	{
		return this.rightSelectionIndex;
	}

	/**
	 * @param rightSelectionIndex
	 *          the rightSelectionIndex to set
	 */
	public void setRightSelectionIndex(final int rightSelectionIndex)
	{
		this.rightSelectionIndex = rightSelectionIndex;
		this.lastClickWasLeft = false;
		redraw();
	}

	/**
	 * @return the leftSelectionIndex
	 */
	public int getLeftSelectionIndex()
	{
		return this.leftSelectionIndex;
	}

	/**
	 * @param leftSelectionIndex
	 *          the leftSelectionIndex to set
	 */
	public void setLeftSelectionIndex(final int leftSelectionIndex)
	{
		this.leftSelectionIndex = leftSelectionIndex;
		this.lastClickWasLeft = true;
		redraw();
	}

	// @Override
	// /**
	// * This method will calculate the default size of the control
	// */
	// public Point computeSize(final int wHint, final int hHint,
	// final boolean changed)
	// {
	// // TODO: implement
	// // get number of colors displayed
	// // calculate what the minimum size has to be in this case
	// // compare with wHint and hHint and compute preferred size
	//
	// // TODO: replace this dummy
	// return new Point(wHint, hHint);
	// }

}
