/**
 * 
 */
package de.digisalt.dsnesds.editors.paletteedit;

import org.eclipse.core.databinding.DataBindingContext;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Slider;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.EditorPart;

import de.digisalt.dsnesds.digui.ColorSpectrumBrowser;
import de.digisalt.dsnesds.digui.PalettePanel;
import de.digisalt.dsnesds.digui.SingleColorSelectionPanel;
import de.digisalt.dsnesds.editors.paletteedit.loader.PaletteLoader;
import de.digisalt.dsnesds.editors.paletteedit.model.Palette;

/**
 * @author flo
 */
public class PaletteEditor extends EditorPart
{
	private DataBindingContext	m_bindingContext;

	public static final String	ID	= "de.digisalt.dsnesds.editors.paletteedit.PaletteEditor";	//$NON-NLS-1$
	private Palette							model;
	private IResource						resource;
	private Composite						container;

	/**
	 * Standard constructor for the editor
	 */
	public PaletteEditor()
	{
	}

	/**
	 * Create contents of the editor part.
	 * 
	 * @param parent
	 */
	@Override
	public void createPartControl(final Composite parent)
	{
		this.container = new Composite(parent, SWT.NONE);
		this.container.setLayout(new GridLayout(8, false));

		Label lblColorPalette = new Label(this.container, SWT.NONE);
		lblColorPalette.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false,
				false, 8, 1));
		lblColorPalette.setText("Color Palette");

		PalettePanel palettePanel = new PalettePanel(this.container, SWT.NONE);
		palettePanel.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 8,
				1));

		Label lblSelectedColor = new Label(this.container, SWT.NONE);
		lblSelectedColor.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false,
				false, 3, 1));
		lblSelectedColor.setText("Selected Color");
		new Label(this.container, SWT.NONE);
		new Label(this.container, SWT.NONE);
		new Label(this.container, SWT.NONE);

		Label lblColorAdjustment = new Label(this.container, SWT.NONE);
		lblColorAdjustment.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false,
				false, 2, 1));
		lblColorAdjustment.setText("Color Adjustment");

		Button button = new Button(this.container, SWT.NONE);
		button.setLayoutData(new GridData(SWT.LEFT, SWT.FILL, false, false, 1, 3));
		button.setText("<");

		SingleColorSelectionPanel selectedColor = new SingleColorSelectionPanel(
				this.container, SWT.NONE);
		selectedColor.setLayoutData(new GridData(SWT.LEFT, SWT.FILL, false, false,
				1, 3));

		Button button_1 = new Button(this.container, SWT.NONE);
		button_1
				.setLayoutData(new GridData(SWT.LEFT, SWT.FILL, false, false, 1, 3));
		button_1.setText(">");
		new Label(this.container, SWT.NONE);
		new Label(this.container, SWT.NONE);
		new Label(this.container, SWT.NONE);

		Label lblR = new Label(this.container, SWT.NONE);
		lblR.setText("R");

		Slider redValue = new Slider(this.container, SWT.NONE);
		redValue.setMaximum(31);
		new Label(this.container, SWT.NONE);
		new Label(this.container, SWT.NONE);
		new Label(this.container, SWT.NONE);

		Label lblG = new Label(this.container, SWT.NONE);
		lblG.setText("G");

		Slider greenValue = new Slider(this.container, SWT.NONE);
		greenValue.setMaximum(31);
		new Label(this.container, SWT.NONE);
		new Label(this.container, SWT.NONE);
		new Label(this.container, SWT.NONE);

		Label lblB = new Label(this.container, SWT.NONE);
		lblB.setText("B");

		Slider blueValue = new Slider(this.container, SWT.NONE);
		blueValue
				.setLayoutData(new GridData(SWT.LEFT, SWT.FILL, false, false, 1, 1));
		blueValue.setMaximum(31);

		Label lblColorSpectrumBrowser = new Label(this.container, SWT.NONE);
		lblColorSpectrumBrowser.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER,
				false, false, 8, 1));
		lblColorSpectrumBrowser.setText("Color Spectrum Browser");

		ColorSpectrumBrowser spectrumBrowser = new ColorSpectrumBrowser(
				this.container, SWT.NONE);
		spectrumBrowser.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true,
				8, 1));

	}

	@Override
	public void init(final IEditorSite site, final IEditorInput input)
			throws PartInitException
	{
		if (!(input instanceof IFileEditorInput))
		{
			throw new PartInitException("Invalid Input: Must be IFileEditorInput");
		}

		// create working copy
		IFile inputFile = ((IFileEditorInput) input).getFile();

		// load the model
		try
		{
			this.model = PaletteLoader.load(inputFile);
		}
		catch (Exception e)
		{
			throw new PartInitException("Couldn't load palette file "
					+ inputFile.getName(), e);
		}

		// store the loaded resource for updates, etc
		this.resource = inputFile;

		// set the editor tab name to the name of the bamf-file currently opened
		setPartName(inputFile.getName());

		setSite(site);
		setInput(input);
	}

	@Override
	public void doSave(final IProgressMonitor monitor)
	{
		// Do the Save operation
	}

	@Override
	public void doSaveAs()
	{
		// Do the Save As operation
	}

	@Override
	public boolean isDirty()
	{
		// TODO implement
		return false;
	}

	@Override
	public boolean isSaveAsAllowed()
	{
		// TODO: implement when we have alllooooooot of time
		return false;
	}

	@Override
	public void setFocus()
	{
		// TODO Auto-generated method stub

	}
}
