/**
 * 
 */
package de.digisalt.dsnesds.editors.paletteedit.loader;

import java.io.File;
import java.io.FileInputStream;

import org.eclipse.core.resources.IFile;

import de.digisalt.dsnesds.core.utils.palette.PaletteUtils;
import de.digisalt.dsnesds.editors.paletteedit.model.Color;
import de.digisalt.dsnesds.editors.paletteedit.model.Palette;

/**
 * @author flo
 * 
 */
public class PaletteLoader
{
	/**
	 * This is the method triggered when trying to load a palette file into the
	 * DOM for the editor
	 * 
	 * @param resource
	 *          the IFile to load from
	 * @return the Palette eObject created
	 * @throws Exception
	 *           may occur if anything goes wrong while loading
	 */
	public static Palette load(final IFile resource) throws Exception
	{
		File in = new File(resource.getLocation().toOSString());
		
		// get filesize to decide which color depth the palette should have
		int numberOfColors = ((int) (in.length()) / 2);
		
		// error handling in case non standard sized palette found
		if (numberOfColors != 256 && numberOfColors != 16 && numberOfColors != 8
				&& numberOfColors != 4 && numberOfColors != 2)
		{
			throw new Exception("The palette with the name " + resource.getName()
					+ " has a non-standard size. It contains " + numberOfColors
					+ " entries.");
		}
		
		// create palette object with all attributes
		Palette pal = new Palette(numberOfColors);
		
		// get byte content of the file (the colors)
		byte[] bytes = new byte[numberOfColors * 2];
		FileInputStream s = new FileInputStream(in);
		s.read(bytes, 0, numberOfColors * 2);
		s.close();
		
		// iterate over the byte array and convert the bytes into color objects
		for (int i = 0; i < numberOfColors; i++)
		{
			byte one = bytes[(i * 2) + 0];
			byte two = bytes[(i * 2) + 1];
			int[] rgb = PaletteUtils.getRawSnesColor(one, two);
			// create color object
			Color color = new Color(rgb[0], rgb[1], rgb[2]);
			// add it to the palette
			pal.setColor(color, i);
		}
		
		// return the palette object created
		return pal;
	}
}
