/**
 * 
 */
package de.digisalt.dsnesds.editors.projectmeta.model;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.part.FileEditorInput;

import de.digisalt.dsnesds.editors.projectmeta.ProjectMetaConstants;

/**
 * @author flo
 * 
 */
public class ProjectMetaModelFactory
{
	/**
	 * This method takes the current editor input and creates the model out of it
	 * 
	 * @param editorInput
	 *          the editor input given
	 * @return the model created from the metadata file
	 */
	public static ProjectMetaModel createModel(final IEditorInput editorInput)
	{
		// the model to be filled
		ProjectMetaModel model = new ProjectMetaModel();
		// the IFile to read from
		IFile metaFile = null;
		
		if (editorInput instanceof FileEditorInput)
		{
			FileEditorInput input = (FileEditorInput) editorInput;
			metaFile = input.getFile();
			IProject project = metaFile.getProject();
			// the wrong file was selected to be opened with our editor
			// --> see which project has been selected
			// --> get project meta file from the IProject
			if (!input.getName().equals(ProjectMetaConstants.projectMetaName))
			{
				IPath projectPath = project.getLocation();
				IPath metaPath = projectPath
						.append(ProjectMetaConstants.projectMetaName);
				metaFile = project.getFile(metaPath);
			}
		}
		// create default model
		if (!metaFile.exists())
		{
			return model;
		}
		
		// read data from metafile and return model
		BufferedReader in = null;
		try
		{
			in = new BufferedReader(new InputStreamReader(metaFile.getContents()));
			String line = "";
			while ((line = in.readLine()) != null)
			{
				// skip lines starting with #
				if (!(line.trim()).startsWith("#"))
				{
					// get key and value
					String key = line.split("=")[0];
					String value = line.split("=")[1];
					// set model values accordingly
					if (key.equals("gamename"))
					{
						model.setGameTitle(value);
					} else if (key.equals("shortname"))
					{
						model.setGameShortname(value);
					} else if (key.equals("timing"))
					{
						model.setTiming(value);
					} else if (key.equals("addressing"))
					{
						model.setAddressing(value);
					} else if (key.equals("carttype"))
					{
						model.setCartridgeType(value);
					} else if (key.equals("romsize"))
					{
						model.setRomSize(value);
					} else if (key.equals("ramsize"))
					{
						model.setRamSize(value);
					} else if (key.equals("country"))
					{
						model.setCountryCode(value);
					} else if (key.equals("licensee"))
					{
						model.setLicensee(value);
					} else if (key.equals("videoformat"))
					{
						model.setVideoFormat(value);
					} else if (key.equals("version"))
					{
						model.setVersion(Integer.valueOf(value));
					}
				}
			}
		} catch (CoreException e)
		{
			e.printStackTrace();
			return model;
		} catch (IOException e)
		{
			e.printStackTrace();
			return model;
		}
		return model;
	}
}
