package de.digisalt.dsnesds.editors.projectmeta.pages;

import org.eclipse.core.databinding.Binding;
import org.eclipse.core.databinding.DataBindingContext;
import org.eclipse.core.databinding.UpdateValueStrategy;
import org.eclipse.core.databinding.beans.PojoObservables;
import org.eclipse.core.databinding.observable.value.IObservableValue;
import org.eclipse.jface.databinding.swt.SWTObservables;
import org.eclipse.jface.fieldassist.ControlDecoration;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.forms.IManagedForm;
import org.eclipse.ui.forms.editor.FormEditor;
import org.eclipse.ui.forms.editor.FormPage;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.ScrolledForm;
import org.eclipse.ui.forms.widgets.Section;
import org.eclipse.wb.swt.ResourceManager;

import de.digisalt.dsnesds.core.util.GraphicsFactory;
import de.digisalt.dsnesds.editors.projectmeta.model.PotentialValues;
import de.digisalt.dsnesds.editors.projectmeta.model.ProjectMetaModel;
import de.digisalt.dsnesds.editors.projectmeta.providers.GenericHashmapToComboContentProvider;
import de.digisalt.dsnesds.editors.projectmeta.providers.GenericStringToComboLabelProvider;
import de.digisalt.dsnesds.editors.projectmeta.validators.GameNameValidator;
import de.digisalt.dsnesds.editors.projectmeta.validators.GameShortnameValidator;

/**
 * @author flo
 * 
 *         The first (and up to now single) editor page for the project metadata
 *         editor.
 */
public class ProjectMetaFormPage extends FormPage
{
	/**
	 * the databinding for the game shortname
	 */
	private Binding gameShortnameBinding;
	/**
	 * the databinding for the game longname
	 */
	private Binding gameLongnameBinding;
	/**
	 * the bindingcontext of this page (contains all databindings and references
	 * to their validation status)
	 */
	@SuppressWarnings("unused")
	private DataBindingContext m_bindingContext;
	/**
	 * the textfield for the game longname
	 */
	private Text txtGametitle;
	/**
	 * the textfield for the game shortname
	 */
	private Text txtShortname;
	
	/**
	 * the pages data model to process on
	 */
	private ProjectMetaModel model = new ProjectMetaModel();
	/**
	 * the decorator showing validation results for the input in the game longname
	 * field
	 */
	private ControlDecoration longnameDecorator;
	/**
	 * the decorator showing validation results for the input in the game
	 * shortname field
	 */
	private ControlDecoration shortnameDecorator;
	/**
	 * the countrycode comboviewer
	 */
	private ComboViewer cvCountrycode;
	/**
	 * the licensee comboviewer
	 */
	private ComboViewer cvLicensee;
	/**
	 * the version spinner
	 */
	private Spinner spinnerVersion;
	/**
	 * the timing comboviewer
	 */
	private ComboViewer cvTiming;
	/**
	 * the addressing comboviewer
	 */
	private ComboViewer cvAddressing;
	/**
	 * the carttype comboviewer
	 */
	private ComboViewer cvCarttype;
	/**
	 * the romsize comboviewer
	 */
	private ComboViewer cvRomsize;
	/**
	 * the ramsize comboviewer
	 */
	private ComboViewer cvRamsize;
	private Combo combo_1;
	private Combo combo_2;
	private Combo combo_3;
	private Combo combo_4;
	private Combo combo_5;
	private Combo combo_6;
	private Combo combo;
	
	/**
	 * Create the form page.
	 * 
	 * @param id
	 *          the id of the page
	 * @param title
	 *          the title to be displayed on the pages selectable tab
	 */
	public ProjectMetaFormPage(final String id, final String title)
	{
		super(id, title);
	}
	
	/**
	 * Create the form page.
	 * 
	 * @param editor
	 *          the editor the page belongs to
	 * @param id
	 *          the id of the page
	 * @param title
	 *          the title to be displayed in the editors tab
	 * @param model
	 *          the editor model to process on
	 * @wbp.parser.constructor
	 * @wbp.eval.method.parameter id "Some id"
	 * @wbp.eval.method.parameter title "Some title"
	 */
	public ProjectMetaFormPage(final FormEditor editor, final String id,
			final String title, final ProjectMetaModel model)
	{
		super(editor, id, title);
		this.model = model;
	}
	
	/**
	 * Create contents of the form.
	 * 
	 * @param managedForm
	 *          the base form every widget of this page will be added to
	 */
	@Override
	protected void createFormContent(final IManagedForm managedForm)
	{
		managedForm.getForm().setImage(
				ResourceManager.getPluginImage("de.digisalt.dsnesds.core",
						"graphics/icons/32/staticData.png"));
		FormToolkit toolkit = managedForm.getToolkit();
		ScrolledForm form = managedForm.getForm();
		form.setText("Project Metadata");
		Composite body = form.getBody();
		toolkit.decorateFormHeading(form.getForm());
		toolkit.paintBordersFor(body);
		managedForm.getForm().getBody().setLayout(new FillLayout(SWT.HORIZONTAL));
		
		SashForm sashForm = new SashForm(managedForm.getForm().getBody(), SWT.NONE);
		sashForm.setBounds(0, 0, 64, 64);
		managedForm.getToolkit().adapt(sashForm);
		managedForm.getToolkit().paintBordersFor(sashForm);
		
		Section sctnNewSection = managedForm.getToolkit().createSection(sashForm,
				Section.DESCRIPTION | Section.TITLE_BAR);
		sctnNewSection
				.setDescription("Project constants without any technical impact on the created game.");
		sctnNewSection.marginWidth = 5;
		sctnNewSection.marginHeight = 5;
		managedForm.getToolkit().paintBordersFor(sctnNewSection);
		sctnNewSection.setText("General Metadata");
		sctnNewSection.setExpanded(true);
		
		Composite composite = managedForm.getToolkit().createComposite(
				sctnNewSection, SWT.NONE);
		managedForm.getToolkit().paintBordersFor(composite);
		sctnNewSection.setClient(composite);
		composite.setLayout(new GridLayout(3, false));
		
		Label lblGameTitle = managedForm.getToolkit().createLabel(composite,
				"Game Title:", SWT.NONE);
		lblGameTitle.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, false,
				2, 1));
		
		this.txtGametitle = managedForm.getToolkit().createText(composite,
				"New Text", SWT.NONE);
		this.txtGametitle.setText("");
		this.txtGametitle.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true,
				false, 1, 1));
		
		this.longnameDecorator = new ControlDecoration(this.txtGametitle, SWT.LEFT
				| SWT.TOP);
		this.longnameDecorator.setDescriptionText("Some description");
		
		Label lblShortname = managedForm.getToolkit().createLabel(composite,
				"Shortname:", SWT.NONE);
		lblShortname.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, false,
				2, 1));
		
		this.txtShortname = managedForm.getToolkit().createText(composite,
				"New Text", SWT.NONE);
		this.txtShortname.setText("");
		this.txtShortname.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true,
				false, 1, 1));
		
		this.shortnameDecorator = new ControlDecoration(this.txtShortname, SWT.LEFT
				| SWT.TOP);
		this.shortnameDecorator.setDescriptionText("Some description");
		
		Label lblCountrycode = managedForm.getToolkit().createLabel(composite,
				"Countrycode:", SWT.NONE);
		lblCountrycode.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false,
				false, 2, 1));
		
		this.cvCountrycode = new ComboViewer(composite, SWT.READ_ONLY);
		this.combo = this.cvCountrycode.getCombo();
		this.combo.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1,
				1));
		managedForm.getToolkit().paintBordersFor(this.combo);
		
		Label lblLicensee = managedForm.getToolkit().createLabel(composite,
				"Licensee:", SWT.NONE);
		lblLicensee.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, false,
				2, 1));
		
		this.cvLicensee = new ComboViewer(composite, SWT.READ_ONLY);
		this.combo_1 = this.cvLicensee.getCombo();
		this.combo_1.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false,
				1, 1));
		managedForm.getToolkit().paintBordersFor(this.combo_1);
		
		@SuppressWarnings("unused")
		Label lblVersion = managedForm.getToolkit().createLabel(composite,
				"Version:", SWT.NONE);
		
		Label label = managedForm.getToolkit().createLabel(composite, "1.",
				SWT.NONE);
		label
				.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
		
		this.spinnerVersion = new Spinner(composite, SWT.BORDER);
		this.spinnerVersion.setMaximum(255);
		this.spinnerVersion.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false,
				false, 1, 1));
		managedForm.getToolkit().adapt(this.spinnerVersion);
		managedForm.getToolkit().paintBordersFor(this.spinnerVersion);
		
		Section sctnNewSection_1 = managedForm.getToolkit().createSection(sashForm,
				Section.DESCRIPTION | Section.TITLE_BAR);
		sctnNewSection_1.setDescription("Technical constants of the game");
		sctnNewSection_1.marginWidth = 5;
		sctnNewSection_1.marginHeight = 5;
		managedForm.getToolkit().paintBordersFor(sctnNewSection_1);
		sctnNewSection_1.setText("Technical Metadata");
		sctnNewSection_1.setExpanded(true);
		
		Composite composite_1 = managedForm.getToolkit().createComposite(
				sctnNewSection_1, SWT.NONE);
		managedForm.getToolkit().paintBordersFor(composite_1);
		sctnNewSection_1.setClient(composite_1);
		composite_1.setLayout(new GridLayout(2, false));
		
		@SuppressWarnings("unused")
		Label lblRomTiming = managedForm.getToolkit().createLabel(composite_1,
				"Rom Timing:", SWT.NONE);
		
		this.cvTiming = new ComboViewer(composite_1, SWT.READ_ONLY);
		this.combo_2 = this.cvTiming.getCombo();
		this.combo_2.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false,
				1, 1));
		managedForm.getToolkit().paintBordersFor(this.combo_2);
		
		@SuppressWarnings("unused")
		Label lblAdressing = managedForm.getToolkit().createLabel(composite_1,
				"Addressing:", SWT.NONE);
		
		this.cvAddressing = new ComboViewer(composite_1, SWT.READ_ONLY);
		this.combo_3 = this.cvAddressing.getCombo();
		this.combo_3.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false,
				1, 1));
		managedForm.getToolkit().paintBordersFor(this.combo_3);
		
		Label lblCartridgeType = managedForm.getToolkit().createLabel(composite_1,
				"Cartridge Type:", SWT.NONE);
		lblCartridgeType.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false,
				false, 1, 1));
		
		this.cvCarttype = new ComboViewer(composite_1, SWT.READ_ONLY);
		this.combo_4 = this.cvCarttype.getCombo();
		this.combo_4.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false,
				1, 1));
		managedForm.getToolkit().paintBordersFor(this.combo_4);
		
		@SuppressWarnings("unused")
		Label lblRomSize = managedForm.getToolkit().createLabel(composite_1,
				"ROM Size:", SWT.NONE);
		
		this.cvRomsize = new ComboViewer(composite_1, SWT.READ_ONLY);
		this.combo_5 = this.cvRomsize.getCombo();
		this.combo_5.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false,
				1, 1));
		managedForm.getToolkit().paintBordersFor(this.combo_5);
		
		@SuppressWarnings("unused")
		Label lblRamSize = managedForm.getToolkit().createLabel(composite_1,
				"RAM Size:", SWT.NONE);
		
		this.cvRamsize = new ComboViewer(composite_1, SWT.READ_ONLY);
		this.combo_6 = this.cvRamsize.getCombo();
		this.combo_6.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false,
				1, 1));
		managedForm.getToolkit().paintBordersFor(this.combo_6);
		sashForm.setWeights(new int[]
		{ 1, 1 });
		
		// own custom methods:
		addListeners();
		// provide content and labelproviders for JFace Viewers
		prepareViewers();
		// init databindings
		this.m_bindingContext = initDataBindings();
	}
	
	/**
	 * sets the inputs for the JFace viewers as well as content and label
	 * providers
	 */
	private void prepareViewers()
	{
		// countrycode comboviewer
		this.cvCountrycode
				.setContentProvider(new GenericHashmapToComboContentProvider());
		this.cvCountrycode
				.setLabelProvider(new GenericStringToComboLabelProvider());
		this.cvCountrycode.setInput(PotentialValues.countrycodeHashmap);
		// this.cvCountrycode.setSelection(new
		// StructuredSelection(this.model.getCountryCode()));
		
		// licensee comboviewer
		this.cvLicensee
				.setContentProvider(new GenericHashmapToComboContentProvider());
		this.cvLicensee.setLabelProvider(new GenericStringToComboLabelProvider());
		this.cvLicensee.setInput(PotentialValues.licenseeHashmap);
		// this.cvLicensee.setSelection(new
		// StructuredSelection(this.model.getLicensee()));
		
		// addressing comboviewer
		this.cvAddressing
				.setContentProvider(new GenericHashmapToComboContentProvider());
		this.cvAddressing.setLabelProvider(new GenericStringToComboLabelProvider());
		this.cvAddressing.setInput(PotentialValues.adressingVector);
		// this.cvAddressing.setSelection(new
		// StructuredSelection(this.model.getAddressing()));
		
		// carttype comboviewer
		this.cvCarttype
				.setContentProvider(new GenericHashmapToComboContentProvider());
		this.cvCarttype.setLabelProvider(new GenericStringToComboLabelProvider());
		this.cvCarttype.setInput(PotentialValues.carttypeHashmap);
		// this.cvCarttype.setSelection(new
		// StructuredSelection(this.model.getCartridgeType()));
		
		// ramsize comboviewer
		this.cvRamsize
				.setContentProvider(new GenericHashmapToComboContentProvider());
		this.cvRamsize.setLabelProvider(new GenericStringToComboLabelProvider());
		this.cvRamsize.setInput(PotentialValues.ramsizesHashmap);
		// this.cvRamsize.setSelection(new
		// StructuredSelection(this.model.getRamSize()));
		
		// romsize comboviewer
		this.cvRomsize
				.setContentProvider(new GenericHashmapToComboContentProvider());
		this.cvRomsize.setLabelProvider(new GenericStringToComboLabelProvider());
		this.cvRomsize.setInput(PotentialValues.romsizeHashmap);
		// this.cvRomsize.setSelection(new
		// StructuredSelection(this.model.getRomSize()));
		
		// timing comboviewer
		this.cvTiming
				.setContentProvider(new GenericHashmapToComboContentProvider());
		this.cvTiming.setLabelProvider(new GenericStringToComboLabelProvider());
		this.cvTiming.setInput(PotentialValues.timingVector);
		// this.cvTiming.setSelection(new
		// StructuredSelection(this.model.getTiming()));
		
		// TODO: make comboviewers sorted
	}
	
	/**
	 * a method adding all component listeners to the widgets
	 */
	private void addListeners()
	{
		this.txtGametitle.addModifyListener(new ModifyListener()
		{
			@Override
			public void modifyText(final ModifyEvent e)
			{
				GraphicsFactory.updateDecoratorWithBindingValidation(
						ProjectMetaFormPage.this.longnameDecorator,
						ProjectMetaFormPage.this.gameLongnameBinding);
			}
			
		});
		this.txtShortname.addModifyListener(new ModifyListener()
		{
			@Override
			public void modifyText(final ModifyEvent e)
			{
				GraphicsFactory.updateDecoratorWithBindingValidation(
						ProjectMetaFormPage.this.shortnameDecorator,
						ProjectMetaFormPage.this.gameShortnameBinding);
			}
		});
	}
	
	protected DataBindingContext initDataBindings()
	{
		DataBindingContext bindingContext = new DataBindingContext();
		//
		IObservableValue txtGametitleObserveTextObserveWidget = SWTObservables
				.observeText(this.txtGametitle, SWT.Modify);
		IObservableValue modelGameTitleObserveValue = PojoObservables.observeValue(
				this.model, "gameTitle");
		UpdateValueStrategy strategy = new UpdateValueStrategy();
		strategy.setAfterGetValidator(new GameNameValidator());
		this.gameLongnameBinding = bindingContext.bindValue(
				txtGametitleObserveTextObserveWidget, modelGameTitleObserveValue,
				strategy, null);
		//
		IObservableValue txtShortnameObserveTextObserveWidget = SWTObservables
				.observeText(this.txtShortname, SWT.Modify);
		IObservableValue modelGameShortnameObserveValue = PojoObservables
				.observeValue(this.model, "gameShortname");
		UpdateValueStrategy strategy_1 = new UpdateValueStrategy();
		strategy_1.setAfterGetValidator(new GameShortnameValidator());
		this.gameShortnameBinding = bindingContext.bindValue(
				txtShortnameObserveTextObserveWidget, modelGameShortnameObserveValue,
				strategy_1, null);
		//
		IObservableValue comboObserveSelectionObserveWidget = SWTObservables
				.observeSelection(this.combo);
		IObservableValue modelCountryCodeObserveValue = PojoObservables
				.observeValue(this.model, "countryCode");
		bindingContext.bindValue(comboObserveSelectionObserveWidget,
				modelCountryCodeObserveValue, new UpdateValueStrategy(
						UpdateValueStrategy.POLICY_NEVER), null);
		//
		IObservableValue combo_3ObserveSelectionObserveWidget = SWTObservables
				.observeSelection(this.combo_3);
		IObservableValue modelAddressingObserveValue = PojoObservables
				.observeValue(this.model, "addressing");
		bindingContext.bindValue(combo_3ObserveSelectionObserveWidget,
				modelAddressingObserveValue, new UpdateValueStrategy(
						UpdateValueStrategy.POLICY_NEVER), null);
		//
		IObservableValue combo_4ObserveSelectionObserveWidget = SWTObservables
				.observeSelection(this.combo_4);
		IObservableValue modelCartridgeTypeObserveValue = PojoObservables
				.observeValue(this.model, "cartridgeType");
		bindingContext.bindValue(combo_4ObserveSelectionObserveWidget,
				modelCartridgeTypeObserveValue, new UpdateValueStrategy(
						UpdateValueStrategy.POLICY_NEVER), null);
		//
		IObservableValue combo_5ObserveSelectionObserveWidget = SWTObservables
				.observeSelection(this.combo_5);
		IObservableValue modelRomSizeObserveValue = PojoObservables.observeValue(
				this.model, "romSize");
		bindingContext.bindValue(combo_5ObserveSelectionObserveWidget,
				modelRomSizeObserveValue, new UpdateValueStrategy(
						UpdateValueStrategy.POLICY_NEVER), null);
		//
		IObservableValue combo_6ObserveSelectionObserveWidget = SWTObservables
				.observeSelection(this.combo_6);
		IObservableValue modelRamSizeObserveValue = PojoObservables.observeValue(
				this.model, "ramSize");
		bindingContext.bindValue(combo_6ObserveSelectionObserveWidget,
				modelRamSizeObserveValue, new UpdateValueStrategy(
						UpdateValueStrategy.POLICY_NEVER), null);
		//
		IObservableValue combo_2ObserveSelectionObserveWidget = SWTObservables
				.observeSelection(this.combo_2);
		IObservableValue modelTimingObserveValue = PojoObservables.observeValue(
				this.model, "timing");
		bindingContext.bindValue(combo_2ObserveSelectionObserveWidget,
				modelTimingObserveValue, new UpdateValueStrategy(
						UpdateValueStrategy.POLICY_NEVER), null);
		//
		IObservableValue combo_1ObserveSelectionObserveWidget = SWTObservables
				.observeSelection(this.combo_1);
		IObservableValue modelLicenseeObserveValue = PojoObservables.observeValue(
				this.model, "licensee");
		bindingContext.bindValue(combo_1ObserveSelectionObserveWidget,
				modelLicenseeObserveValue, new UpdateValueStrategy(
						UpdateValueStrategy.POLICY_NEVER), null);
		//
		IObservableValue spinnerVersionObserveSelectionObserveWidget = SWTObservables
				.observeSelection(this.spinnerVersion);
		IObservableValue modelVersionObserveValue = PojoObservables.observeValue(
				this.model, "version");
		bindingContext.bindValue(spinnerVersionObserveSelectionObserveWidget,
				modelVersionObserveValue, null, null);
		//
		return bindingContext;
	}
}
