/**
 * 
 */
package de.digisalt.dsnesds.tileeditor;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.QualifiedName;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.custom.StackLayout;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.ToolBar;
import org.eclipse.swt.widgets.ToolItem;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.EditorPart;
import org.eclipse.wb.swt.ResourceManager;

import de.digisalt.dsnesds.core.DsnesdsConstants;
import de.digisalt.dsnesds.digui.DoubleColorSelectionPanel;
import de.digisalt.dsnesds.digui.PalettePanel;

/**
 * @author flo
 */
public class TileEditor extends EditorPart
{

	public static final String	ID	= "de.digisalt.dsnesds.tileeditor.TileEditor";	//$NON-NLS-1$

	private IResource						resource;

	private ToolItem						tltm2Colors;

	private ToolItem						tltm4Colors;

	private ToolItem						tltm8Colors;

	private ToolItem						tltm16Colors;

	private ToolItem						tltm256Colors;

	private ToolItem						tltmMode7;

	public TileEditor()
	{
	}

	/**
	 * Create contents of the editor part.
	 * 
	 * @param parent
	 */
	@Override
	public void createPartControl(final Composite parent)
	{
		Composite container = new Composite(parent, SWT.NONE);
		container.setLayout(new GridLayout(11, false));

		TwoColumnToolbar tbToolSelection = new TwoColumnToolbar(container, SWT.NONE);
		tbToolSelection.setLayoutData(new GridData(SWT.LEFT, SWT.FILL, false,
				false, 1, 1));

		ScrolledComposite scrolledComposite = new ScrolledComposite(container,
				SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL);
		scrolledComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false,
				false, 10, 6));
		scrolledComposite.setExpandHorizontal(true);
		scrolledComposite.setExpandVertical(true);

		Label label = new Label(container, SWT.HORIZONTAL);
		label.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, true, 1, 1));

		Composite toolOptionsBaseComposite = new Composite(container, SWT.NONE);
		toolOptionsBaseComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL,
				false, false, 1, 1));
		toolOptionsBaseComposite.setLayout(new StackLayout());

		Composite pencilOptionsComposite = new Composite(toolOptionsBaseComposite,
				SWT.NONE);

		ToolBar pencilOptions = new ToolBar(pencilOptionsComposite, SWT.FLAT
				| SWT.RIGHT | SWT.VERTICAL);
		pencilOptions.setSize(80, 64);

		ToolItem toolScope1 = new ToolItem(pencilOptions, SWT.RADIO);
		toolScope1.setImage(ResourceManager.getPluginImage(
				"de.digisalt.dsnesds.editors.tileeditor",
				"graphics/opt_pencilSingle.png"));

		ToolItem toolScope2 = new ToolItem(pencilOptions, SWT.RADIO);
		toolScope2.setImage(ResourceManager.getPluginImage(
				"de.digisalt.dsnesds.editors.tileeditor",
				"graphics/opt_pencilMulti.png"));

		Composite lineOptionsComposite = new Composite(toolOptionsBaseComposite,
				SWT.NONE);

		ToolBar lineOptions = new ToolBar(lineOptionsComposite, SWT.FLAT
				| SWT.RIGHT);
		lineOptions.setSize(80, 64);

		Composite slurpOptionsComposite = new Composite(toolOptionsBaseComposite,
				SWT.NONE);

		ToolBar slurpOptions = new ToolBar(slurpOptionsComposite, SWT.FLAT
				| SWT.RIGHT);
		slurpOptions.setSize(80, 64);

		Composite rubberOptionsComposite = new Composite(toolOptionsBaseComposite,
				SWT.NONE);

		ToolBar rubberOptions = new ToolBar(rubberOptionsComposite, SWT.FLAT
				| SWT.RIGHT);
		rubberOptions.setSize(80, 64);

		Composite fillOptionsComposite = new Composite(toolOptionsBaseComposite,
				SWT.NONE);

		ToolBar fillOptions = new ToolBar(fillOptionsComposite, SWT.FLAT
				| SWT.RIGHT);
		fillOptions.setSize(80, 64);

		Composite markOptionsComposite = new Composite(toolOptionsBaseComposite,
				SWT.NONE);

		ToolBar markOptions = new ToolBar(markOptionsComposite, SWT.FLAT
				| SWT.RIGHT);
		markOptions.setSize(80, 64);

		Composite rectOptionsComposite = new Composite(toolOptionsBaseComposite,
				SWT.NONE);

		ToolBar rectOptions = new ToolBar(rectOptionsComposite, SWT.FLAT
				| SWT.RIGHT);
		rectOptions.setSize(80, 64);

		Composite ovalOptionsComposite = new Composite(toolOptionsBaseComposite,
				SWT.NONE);

		ToolBar ovalOptions = new ToolBar(ovalOptionsComposite, SWT.FLAT
				| SWT.RIGHT);
		ovalOptions.setSize(80, 64);

		Composite numberOptionsComposite = new Composite(toolOptionsBaseComposite,
				SWT.NONE);

		ToolBar numberOptions = new ToolBar(numberOptionsComposite, SWT.FLAT
				| SWT.RIGHT);
		numberOptions.setSize(80, 64);

		Composite textOptionsComposite = new Composite(toolOptionsBaseComposite,
				SWT.NONE);

		ToolBar textOptions = new ToolBar(textOptionsComposite, SWT.FLAT
				| SWT.RIGHT);
		textOptions.setSize(80, 64);

		Composite magOptionsComposite = new Composite(toolOptionsBaseComposite,
				SWT.NONE);

		Composite modOptionsComposite = new Composite(toolOptionsBaseComposite,
				SWT.NONE);

		Label label_1 = new Label(container, SWT.HORIZONTAL);
		label_1
				.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, true, 1, 1));

		ToolBar tbColorDepth = new ToolBar(container, SWT.FLAT | SWT.RIGHT
				| SWT.VERTICAL);
		tbColorDepth.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false,
				1, 1));

		tltm2Colors = new ToolItem(tbColorDepth, SWT.RADIO);
		tltm2Colors.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent arg0)
			{
				switchEncodingTo(2);
			}
		});
		tltm2Colors.setText("2 colors");

		tltm4Colors = new ToolItem(tbColorDepth, SWT.RADIO);
		tltm4Colors.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent arg0)
			{
				switchEncodingTo(4);
			}
		});

		tltm4Colors.setText("4 colors");

		tltm8Colors = new ToolItem(tbColorDepth, SWT.RADIO);
		tltm8Colors.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent arg0)
			{
				switchEncodingTo(8);
			}
		});
		tltm8Colors.setText("8 colors");

		tltm16Colors = new ToolItem(tbColorDepth, SWT.RADIO);
		tltm16Colors.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent arg0)
			{
				switchEncodingTo(16);
			}
		});
		tltm16Colors.setText("16 colors");

		tltm256Colors = new ToolItem(tbColorDepth, SWT.RADIO);
		tltm256Colors.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent arg0)
			{
				switchEncodingTo(256);
			}
		});
		tltm256Colors.setText("256 colors");

		tltmMode7 = new ToolItem(tbColorDepth, SWT.RADIO);
		tltmMode7.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent arg0)
			{
				switchEncodingTo(512);
			}
		});
		tltmMode7.setText("Mode 7 (256)");

		Label label_2 = new Label(container, SWT.HORIZONTAL);
		label_2
				.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, true, 1, 1));

		DoubleColorSelectionPanel selectedColors = new DoubleColorSelectionPanel(
				container, SWT.NONE);
		selectedColors.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false,
				1, 3));

		PalettePanel composite = new PalettePanel(container, SWT.NONE);
		GridData gd_composite = new GridData(SWT.LEFT, SWT.CENTER, false, false, 1,
				3);
		gd_composite.widthHint = 161;
		composite.setLayoutData(gd_composite);

		Label label_3 = new Label(container, SWT.NONE);
		label_3.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1,
				3));

		ComboViewer comboViewer = new ComboViewer(container, SWT.NONE);
		Combo combo = comboViewer.getCombo();
		GridData gd_combo = new GridData(SWT.FILL, SWT.CENTER, false, false, 4, 1);
		gd_combo.widthHint = 165;
		combo.setLayoutData(gd_combo);

		Button btnRefresh = new Button(container, SWT.FLAT);
		btnRefresh.setImage(ResourceManager.getPluginImage(
				"de.digisalt.dsnesds.editors.tileeditor", "graphics/refresh.png"));
		btnRefresh.setLayoutData(new GridData(SWT.LEFT, SWT.BOTTOM, false, false,
				1, 1));

		Label label_6 = new Label(container, SWT.NONE);
		label_6
				.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 3));

		Label lblRows = new Label(container, SWT.NONE);
		lblRows.setText("Rows:");

		Spinner spinner_1 = new Spinner(container, SWT.BORDER);
		spinner_1.setMinimum(1);

		Label label_5 = new Label(container, SWT.HORIZONTAL);
		label_5
				.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, false, 5, 1));
		new Label(container, SWT.NONE);
		new Label(container, SWT.NONE);

		Label lblZoomFactor = new Label(container, SWT.NONE);
		lblZoomFactor.setText("Zoom Factor:");

		Spinner spinner = new Spinner(container, SWT.BORDER);
		spinner.setMinimum(1);

		Label label_4 = new Label(container, SWT.NONE);
		label_4.setLayoutData(new GridData(SWT.CENTER, SWT.CENTER, false, false, 1,
				1));

		Button btnGridEnabled = new Button(container, SWT.CHECK);
		btnGridEnabled.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, true,
				false, 1, 1));
		btnGridEnabled.setText("Grid enabled");
		new Label(container, SWT.NONE);

		Label lblColumns = new Label(container, SWT.NONE);
		lblColumns.setText("Columns:");

		Spinner spinner_2 = new Spinner(container, SWT.BORDER);
		spinner_2.setMinimum(1);

		try
		{
			processMetadata();
		}
		catch (CoreException e)
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	@Override
	public void setFocus()
	{
		// Set the focus
	}

	@Override
	public void doSave(final IProgressMonitor monitor)
	{
		// Do the Save operation
	}

	@Override
	public void doSaveAs()
	{
		// Do the Save As operation
	}

	@Override
	public void init(final IEditorSite site, final IEditorInput input)
			throws PartInitException
	{
		if (!(input instanceof IFileEditorInput))
		{
			throw new PartInitException("Invalid Input: Must be IFileEditorInput");
		}

		// create working copy
		IFile inputFile = ((IFileEditorInput) input).getFile();

		// load the model
		try
		{
			// TODO reactivate as soon as we have a proper model object
			// this.model = PaletteLoader.load(inputFile);
		}
		catch (Exception e)
		{
			throw new PartInitException("Couldn't load palette file "
					+ inputFile.getName(), e);
		}

		// store the loaded resource for updates, etc
		this.resource = inputFile;

		// set the editor tab name to the name of the bamf-file currently opened
		setPartName(inputFile.getName());

		setSite(site);
		setInput(input);
	}

	// get metadata from the tileset file and set some editor attributes
	// appropriately
	private void processMetadata() throws CoreException
	{
		// set color depth/encoding
		String encoding = resource.getPersistentProperty(new QualifiedName(
				DsnesdsConstants.namespace_tilesetFile,
				DsnesdsConstants.tileset_preferredEncoding));
		switch (encoding)
		{
		case "2 Colors (1bpp)":
		{
			tltm2Colors.setSelection(true);
			break;
		}
		case "4 Colors (2bpp)":
		{
			tltm4Colors.setSelection(true);
			break;
		}
		case "8 Colors (3bpp)":
		{
			tltm8Colors.setSelection(true);
			break;
		}
		case "16 Colors (4bpp)":
		{
			tltm16Colors.setSelection(true);
			break;
		}
		case "256 Colors (8bpp)":
		{
			tltm256Colors.setSelection(true);
			break;
		}
		case "Mode7 (8bpp)":
		{
			tltmMode7.setSelection(true);
			break;
		}
		default:
			tltm2Colors.setSelection(true);
			break;
		}
		// TODO set number of columns
		// TODO set preferred palette

	}

	@Override
	public boolean isDirty()
	{
		return false;
	}

	@Override
	public boolean isSaveAsAllowed()
	{
		return false;
	}

	/**
	 * Changes the property for encoding according to user selection and updates
	 * the UI appropriately
	 * 
	 * @param i
	 */
	private void switchEncodingTo(int i)
	{
		String newencodingProperty = "";
		switch (i)
		{
		case 2:
		{
			newencodingProperty = "2 Colors (1bpp)";
			break;
		}
		case 4:
		{
			newencodingProperty = "4 Colors (2bpp)";
			break;
		}
		case 8:
		{
			newencodingProperty = "8 Colors (3bpp)";
			break;
		}
		case 16:
		{
			newencodingProperty = "16 Colors (4bpp)";
			break;
		}
		case 256:
		{
			newencodingProperty = "256 Colors (8bpp)";
			break;
		}
		case 512:
		{
			newencodingProperty = "Mode7 (8bpp)";
			break;
		}
		default:
			break;
		}
		try
		{
			resource.setPersistentProperty(new QualifiedName(
					DsnesdsConstants.namespace_tilesetFile,
					DsnesdsConstants.tileset_preferredEncoding), newencodingProperty);
		}
		catch (CoreException e)
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		// TODO change editor views (react on ecoding instead of just saving editor
		// metadata)
	}

}
