/**
 * 
 */
package de.digisalt.dsnesds.tileeditor;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.ToolBar;
import org.eclipse.swt.widgets.ToolItem;
import org.eclipse.wb.swt.ResourceManager;

/**
 * @author flo This class was necessary due to a bug in SWT in GTK/Linux. The
 *         problem is that wrapping of toolbar items is not properly possible.
 *         Therefore having two columns of toolbar items next to each other is
 *         not possible by default. This works around the problem by creating
 *         two separate toolbars next to each other which are automatically
 *         linked so that only one toolitem from either of the toolbars can be
 *         selected.
 */
public class TwoColumnToolbar extends Composite
{

	private final ToolBar	rightToolbar;
	private final ToolBar	leftToolbar;

	/**
	 * @param parent
	 * @param style
	 */
	public TwoColumnToolbar(final Composite parent, final int style)
	{
		super(parent, style);
		setLayout(new FillLayout(SWT.HORIZONTAL));

		this.leftToolbar = new ToolBar(this, SWT.FLAT | SWT.RIGHT | SWT.VERTICAL);

		ToolItem tltmPencil = new ToolItem(this.leftToolbar, SWT.RADIO);
		tltmPencil.setSelection(true);
		tltmPencil.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(final SelectionEvent e)
			{
				setSelection((ToolItem) e.getSource());
			}
		});
		tltmPencil.setImage(ResourceManager.getPluginImage(
				"de.digisalt.dsnesds.editors.tileeditor", "graphics/tool_pencil.png"));
		ToolItem tltmSlurp = new ToolItem(this.leftToolbar, SWT.RADIO);
		tltmSlurp.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(final SelectionEvent e)
			{
				setSelection((ToolItem) e.getSource());
			}
		});
		tltmSlurp.setImage(ResourceManager.getPluginImage(
				"de.digisalt.dsnesds.editors.tileeditor", "graphics/tool_slurp.png"));
		ToolItem tltmFill = new ToolItem(this.leftToolbar, SWT.RADIO);
		tltmFill.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(final SelectionEvent e)
			{
				setSelection((ToolItem) e.getSource());
			}
		});
		tltmFill.setImage(ResourceManager.getPluginImage(
				"de.digisalt.dsnesds.editors.tileeditor", "graphics/tool_fill.png"));
		ToolItem tltmRect = new ToolItem(this.leftToolbar, SWT.RADIO);
		tltmRect.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(final SelectionEvent e)
			{
				setSelection((ToolItem) e.getSource());
			}
		});
		tltmRect.setImage(ResourceManager.getPluginImage(
				"de.digisalt.dsnesds.editors.tileeditor", "graphics/tool_rect.png"));
		ToolItem tltmEnum = new ToolItem(this.leftToolbar, SWT.RADIO);
		tltmEnum.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(final SelectionEvent e)
			{
				setSelection((ToolItem) e.getSource());
			}
		});
		tltmEnum.setImage(ResourceManager.getPluginImage(
				"de.digisalt.dsnesds.editors.tileeditor", "graphics/tool_number.png"));
		ToolItem tltmMagni = new ToolItem(this.leftToolbar, SWT.RADIO);
		tltmMagni.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(final SelectionEvent e)
			{
				setSelection((ToolItem) e.getSource());
			}
		});
		tltmMagni.setImage(ResourceManager.getPluginImage(
				"de.digisalt.dsnesds.editors.tileeditor", "graphics/tool_zoom.png"));

		this.rightToolbar = new ToolBar(this, SWT.FLAT | SWT.RIGHT | SWT.VERTICAL);

		ToolItem tltmLine = new ToolItem(this.rightToolbar, SWT.RADIO);
		tltmLine.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(final SelectionEvent e)
			{
				setSelection((ToolItem) e.getSource());
			}
		});
		tltmLine.setImage(ResourceManager.getPluginImage(
				"de.digisalt.dsnesds.editors.tileeditor", "graphics/tool_line.png"));
		ToolItem tltmRubber = new ToolItem(this.rightToolbar, SWT.RADIO);
		tltmRubber.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(final SelectionEvent e)
			{
				setSelection((ToolItem) e.getSource());
			}
		});
		tltmRubber.setImage(ResourceManager.getPluginImage(
				"de.digisalt.dsnesds.editors.tileeditor", "graphics/tool_rubber.png"));
		ToolItem tltmMark = new ToolItem(this.rightToolbar, SWT.RADIO);
		tltmMark.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(final SelectionEvent e)
			{
				setSelection((ToolItem) e.getSource());
			}
		});
		tltmMark.setImage(ResourceManager.getPluginImage(
				"de.digisalt.dsnesds.editors.tileeditor", "graphics/tool_mark.png"));
		ToolItem tltmOval = new ToolItem(this.rightToolbar, SWT.RADIO);
		tltmOval.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(final SelectionEvent e)
			{
				setSelection((ToolItem) e.getSource());
			}
		});
		tltmOval.setImage(ResourceManager.getPluginImage(
				"de.digisalt.dsnesds.editors.tileeditor", "graphics/tool_oval.png"));
		ToolItem tltmText = new ToolItem(this.rightToolbar, SWT.RADIO);
		tltmText.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(final SelectionEvent e)
			{
				setSelection((ToolItem) e.getSource());
			}
		});
		tltmText.setImage(ResourceManager.getPluginImage(
				"de.digisalt.dsnesds.editors.tileeditor", "graphics/tool_text.png"));

	}

	/**
	 * If a new toolitem was selected, the old selection needs to be deactivated
	 * 
	 * @param t
	 *          the toolitem which was selected by the user
	 */
	private void setSelection(final ToolItem t)
	{
		// if there was any selection from the other toolbar (not the one the
		// currently selected element resides in), we have to deactivate it manually
		ToolItem[] items = null;
		if (t.getParent().equals(this.leftToolbar))
		{
			items = this.rightToolbar.getItems();
		}
		else
		{
			items = this.leftToolbar.getItems();
		}
		for (ToolItem i : items)
		{
			if (i.getSelection())
			{
				i.setSelection(false);
			}
		}
	}
}
